package Cli;

import java.util.ArrayList;
import java.util.StringTokenizer;

import Commands.Login;
import Commands.AddDishRestaurantMenu;
import Commands.AddDishToMeal;
import Commands.AddItemToOrder;
import Commands.AssociateCard;
import Commands.CreateMeal;
import Commands.CreateOrder;
import Commands.EndOrder;
import Commands.EndOrderAt;
import Commands.Help;
import Commands.Logout;
import Commands.OffDuty;
import Commands.OnDuty;
import Commands.RegisterCourier;
import Commands.RegisterCustomer;
import Commands.RegisterManager;
import Commands.RegisterRestaurant;
import Commands.RemoveSpecialOffer;
import Commands.RenameMeal;
import Commands.RunTest;
import Commands.SetDeliveryPolicy;
import Commands.SetProfitPolicy;
import Commands.SetSpecialOffer;
import Commands.ShowCourier;
import Commands.ShowCourierOrdered;
import Commands.ShowCustomer;
import Commands.ShowMeal;
import Commands.ShowMenuItem;
import Commands.ShowProfitBetween;
import Commands.ShowRestaurantTop;
import Commands.ShowTotalProfit;
import Core.ActiveUserContext;
import Core.MyFoodora;
import Exception.ExceptionUnknownDishType;
import Exception.ExceptionUnknownMealType;
import Exception.ExceptionUnknownStartegyType;
import Exception.ExceptionUnknownTypeOfUser;
import User.Customer;

/**
 * Clui controller, create a context allowing cli into foodora system.
 * Each command is parsed, arguments are verified through regex and the number of argument.
 * 
 * @author Nicolas FLEY et Corentin RAVOUX
 * 
 */

public class Clui {
	
	private MyFoodora foodora;
	private ActiveUserContext activeUser;
	private ArrayList<Command> listOfCommand = new ArrayList<Command>();
	
	
	public Clui(MyFoodora _foodora, ActiveUserContext _activeUser) {
		super();
		this.foodora = _foodora;
		this.activeUser = _activeUser;
		this.listOfCommand.add(new AddDishRestaurantMenu());
		this.listOfCommand.add(new AddDishToMeal());
		this.listOfCommand.add(new RegisterCourier());
		this.listOfCommand.add(new AddItemToOrder());
		this.listOfCommand.add(new AssociateCard());
		this.listOfCommand.add(new CreateMeal());
		this.listOfCommand.add(new CreateOrder());
		this.listOfCommand.add(new EndOrder());
		this.listOfCommand.add(new EndOrderAt());
		this.listOfCommand.add(new Help());
		this.listOfCommand.add(new Login());
		this.listOfCommand.add(new Logout());
		this.listOfCommand.add(new OffDuty());
		this.listOfCommand.add(new OnDuty());
		this.listOfCommand.add(new RegisterCourier());
		this.listOfCommand.add(new RegisterCustomer());
		this.listOfCommand.add(new RegisterManager());
		this.listOfCommand.add(new RegisterRestaurant());
		this.listOfCommand.add(new RemoveSpecialOffer());
		this.listOfCommand.add(new RenameMeal());
		this.listOfCommand.add(new RunTest());
		this.listOfCommand.add(new SetDeliveryPolicy());
		this.listOfCommand.add(new SetProfitPolicy());
		this.listOfCommand.add(new SetSpecialOffer());
		this.listOfCommand.add(new ShowCourier());
		this.listOfCommand.add(new ShowCourierOrdered());
		this.listOfCommand.add(new ShowCustomer());
		this.listOfCommand.add(new ShowMeal());
		this.listOfCommand.add(new ShowMenuItem());
		this.listOfCommand.add(new ShowProfitBetween());
		this.listOfCommand.add(new ShowRestaurantTop());
		this.listOfCommand.add(new ShowTotalProfit());
	}

	public Command getCommand(String commandName) {
		for(Command c : this.listOfCommand){
			if(c.isOneWanted(commandName))
				return c;
		}
		return null;
	}

	public void executeCommand(String command){
		if(!(command.equals("") || command.substring(0, 1).equals("#"))){
			StringTokenizer st = new StringTokenizer(command);
			String commandName = st.nextToken();
			ArrayList<String> arguments = new ArrayList<String>();
			while (st.hasMoreTokens()) {
				arguments.add(st.nextToken());
		    }
			
			Command commandAsked;
			commandAsked = getCommand(commandName);
			if(commandAsked != null){
				try{
					if(commandAsked.isGoodArgument(arguments)){
						try {
							commandAsked.execute(arguments, foodora, activeUser);
						} catch (NumberFormatException | ExceptionUnknownMealType | ExceptionUnknownDishType
								| ExceptionUnknownStartegyType | ExceptionUnknownTypeOfUser | java.lang.NullPointerException e) {
							// TODO Auto-generated catch block
							System.out.println("ERROR with command "+commandName+ " : " + e.getMessage());
							//e.printStackTrace();
						}
					}else{
						System.out.println("Error in the arguments.");
					}
				}catch (errorWrongNumberOfParams ex) {
				     System.out.println(ex);
				}
				catch (errorWrongFormatValue ex) {
				     System.out.println(ex);
				}
			}else{
				System.out.println("Unknown command.");
			}
			
			// lifeTimeFunction
			if(activeUser.getUser() instanceof Customer){
				Customer c = (Customer) activeUser.getUser();
				c.checkForPendingOffer();
			}
			
		}
	}
	
	public void launchClui(){
		System.out.println("Welcome to MyFoodora system, type help <> in order to get all available commands");
		while(true){
			String command = Input.string("===================\n"
										+ "Type your command : ");
			executeCommand(command);
		}
	}
	
}
