package Cli;

import java.util.ArrayList;

import Core.ActiveUserContext;
import Core.MyFoodora;
import Exception.ExceptionUnknownDishType;
import Exception.ExceptionUnknownMealType;
import Exception.ExceptionUnknownStartegyType;
import Exception.ExceptionUnknownTypeOfUser;

/**
 * Describes how all command are, composed by a commandName and a listOfArgs,
 * It allow to verify if each argument is correct, and if all arguments are there.
 * Commands execute should verify if the user is connected and have the good instance value
 *  by using the ActiveUserContext variable.
 *
 * @author Nicolas FLEY et Corentin RAVOUX
 *
 */

public abstract class Command {
	private String commandName;
	private Token [] listOfArgs;
	
	public Command(String _commandName, Token [] _listOfArgs){
		commandName = _commandName;
		listOfArgs = _listOfArgs;
	}
	
	public boolean isOneWanted(String firstToken){
		if(firstToken.toLowerCase().equals(commandName.toLowerCase()))
			return true;
		return false;
	}
	
	public boolean isGoodArgument(ArrayList<String> arguments) throws errorWrongNumberOfParams,errorWrongFormatValue{
		if(arguments.size() != listOfArgs.length){
			throw new errorWrongNumberOfParams("Error : "+ commandName +" : "+Integer.toString(arguments.size())+" parameter insteed of "+Integer.toString(listOfArgs.length));
		}
		int i = 0;
		for (String arg : arguments) {
			if(listOfArgs[i].isTockenCorrect(arg))
				i++;
			else
				throw new errorWrongFormatValue("Error : "+ commandName +" : " + i + " arg is wrong format");
		}
		 // trow une erreur
		return true;
	}
	
	public abstract void execute(ArrayList<String> arguments, MyFoodora foodora, ActiveUserContext activeUser) throws ExceptionUnknownMealType, NumberFormatException, ExceptionUnknownDishType, ExceptionUnknownStartegyType, ExceptionUnknownTypeOfUser;
	
}
