package Core;

import java.util.ArrayList;
import java.util.Collections;
import Exception.ExceptionUnknownStartegyType;
import Order.HistoricOrder;
import Order.Order;
import StrategyProfit.ContextTargetProfitStrategy;
import User.Courier;
import User.Customer;
import User.ListUser;
import User.Manager;
import User.Restaurant;
import User.User;
import DeliveryStrategy.ContextDeliveryStrategy;

/**
 * Heart of the project, concatenate all the functionalities this project
 * must have. This class should be unique and contain the entire foodora system.
 * It also implement a lot of method which allow users to modify itself.
 * 
 * @author Nicolas FLEY et Corentin RAVOUX
 *
 */


public class MyFoodora {
	
	public static final int UNKNOWN = 0;
	public static final int COURIER = 1;
	public static final int RESTAURANT = 2;
	public static final int CUSTOMER = 3;
	public static final int MANAGER = 4;
	
	private double serviceFee;
	private double markupPercentage;
	private double deliveryCost;
	
	private ContextTargetProfitStrategy contextTargetProfitStrategy;
	private ContextDeliveryStrategy contextDeliveryPolicy;
	
	private ListUser<User> globalListUser;
	private ListUser<Restaurant> listRestaurant;
	private ListUser<Customer> listCustomer;
	private ListUser<Manager> listManager;
	private ListUser<Courier> listCourier;

	private ArrayList<Order> listPendingOrder;
	
	private HistoricOrder globalHistoric;
	
	/** CONSTRUCTOR 
	 * @throws ExceptionUnknownStartegyType **/
	
	public MyFoodora(){
		try {
			this.contextTargetProfitStrategy = new ContextTargetProfitStrategy("DeliveryCost");
		} catch (ExceptionUnknownStartegyType e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		try {
			this.contextDeliveryPolicy = new ContextDeliveryStrategy("FairOccupation");
		} catch (ExceptionUnknownStartegyType e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		this.listRestaurant = new ListUser<Restaurant>();
		this.listCustomer = new ListUser<Customer>();
		this.listManager = new ListUser<Manager>();
		this.listCourier = new ListUser<Courier>();
		this.globalListUser = new ListUser<User>();
		this.globalHistoric = new HistoricOrder();
		this.listPendingOrder = new ArrayList<Order>();
	}
	
	/** CUSTOM **/
	public User getUserByLogin(String usernameTest, String passwordTest){
		User u = this.getListCustomer().getUserByLogin(usernameTest, passwordTest);
		if(u != null)
			return u;
		u = this.getListCourier().getUserByLogin(usernameTest, passwordTest);
		if(u != null)
			return u;
		u = this.getListRestaurant().getUserByLogin(usernameTest, passwordTest);
		if(u != null)
			return u;
		u = this.getListManager().getUserByLogin(usernameTest, passwordTest);
		if(u != null)
			return u;
		return null;
	}
	
	public User getUserByUsername(String username){
		for(User user : this.getGlobalListUser().getList()){
			if(username.equals(user.getUsername())){
				return user;
			}
		}
		System.out.println("\n /!\\ WARNING : " + username + " is not in user database.");
		return null;	
	}
	
	public User getUserByName(String username, String name){
		for(User user : this.getGlobalListUser().getList()){
			if(username.equals(user.getUsername()) && name.equals(user.getName())){
				return user;
			}
		}
		return null;	
	}
	
	
	public void addUser(User u){
		this.globalListUser.addUser(u);
	}
	
	public void addCustomer(Customer c){
		this.listCustomer.addUser(c);
	}
	
	public void addRestaurant(Restaurant r){
		this.listRestaurant.addUser(r);
	}
	
	public void addCourier(Courier c){
		this.listCourier.addUser(c);
	}
	
	public void addManager(Manager m){
		this.listManager.addUser(m);
	}
	
	public void removeWithID(int typeOfUser,long id){
		User userToRemove = null;
		for (User user : this.globalListUser.getList()){
			if(user.getId() == id){
				userToRemove = user;
			}
		}
		if(userToRemove != null){
			this.globalListUser.removeUser(userToRemove);
		}
		if (typeOfUser == CUSTOMER){
			Customer customerToRemove = null;
				for (Customer customer : this.listCustomer.getList()){
					if(customer.getId() == id){
						customerToRemove = customer;
					}
			}
			if(customerToRemove != null){
				this.listCustomer.removeUser(customerToRemove);
			}
		}
		else if (typeOfUser == RESTAURANT){
			Restaurant restaurantToRemove = null;
				for (Restaurant restaurant : this.listRestaurant.getList()){
					if(restaurant.getId() == id){
						restaurantToRemove = restaurant;
					}
			
			}
				if(restaurantToRemove != null){
					this.listRestaurant.removeUser(restaurantToRemove);

				}
		}
		else if (typeOfUser == MANAGER){
			Manager managerToRemove = null;
			for (Manager manager : this.listManager.getList()){
				if(manager.getId() == id){
					managerToRemove = manager;
				}
		
			}
			if(managerToRemove != null){
				this.listManager.removeUser(managerToRemove);

			}
		}
		else if (typeOfUser == COURIER){
			Courier courierToRemove = null;
			for (Courier courier : this.listCourier.getList()){
				if(courier.getId() == id){
					courierToRemove = courier;
				}
		
			}
			if(courierToRemove != null){
				this.listCourier.removeUser(courierToRemove);
			}
		}
	}
	
	/** GENERIC GETTER AND SETTER **/
	
	
	
	public double getServiceFee() {
		return serviceFee;
	}
	public ListUser<User> getGlobalListUser() {
		return globalListUser;
	}

	public void setGlobalListUser(ListUser<User> globalListUser) {
		this.globalListUser = globalListUser;
	}

	public void setListRestaurant(ListUser<Restaurant> listRestaurant) {
		this.listRestaurant = listRestaurant;
	}

	public void setListCustomer(ListUser<Customer> listCustomer) {
		this.listCustomer = listCustomer;
	}

	public void setListManager(ListUser<Manager> listManager) {
		this.listManager = listManager;
	}

	public ListUser<Restaurant> getListRestaurant() {
		return listRestaurant;
	}
	public ListUser<Customer> getListCustomer() {
		return listCustomer;
	}
	public ListUser<Manager> getListManager() {
		return listManager;
	}
	public ListUser<Courier> getListCourier() {
		return listCourier;
	}
	public ListUser<Courier> getListCourierOrdered(String type) {
		Collections.sort(listCourier.getList());
		switch(type){
			case "ASC":
			break;
			case "DESC":
				Collections.reverse(listCourier.getList());
			break;
			default:
				System.out.println("Warning, "+type+" is unkown, only ASC & DESC are known.");
		}
		return listCourier;
	}
	public HistoricOrder getGlobalHistoric() {
		return globalHistoric;
	}
	
	

	public ContextTargetProfitStrategy getContextTargetProfitStrategy() {
		return contextTargetProfitStrategy;
	}

	public ContextDeliveryStrategy getContextDeliveryPolicy() {
		return contextDeliveryPolicy;
	}

	public void setContextTargetProfitStrategy(ContextTargetProfitStrategy contextTargetProfitStrategy) {
		this.contextTargetProfitStrategy = contextTargetProfitStrategy;
	}

	public void setContextDeliveryPolicy(ContextDeliveryStrategy contextDeliveryPolicy) {
		this.contextDeliveryPolicy = contextDeliveryPolicy;
	}

	public void setListCourier(ListUser<Courier> listCourier) {
		this.listCourier = listCourier;
	}

	public void setServiceFee(double serviceFee) {
		this.serviceFee = serviceFee;
	}
	public double getMarkupPercentage() {
		return markupPercentage;
	}
	public void setMarkupPercentage(double markupPercentage) {
		this.markupPercentage = markupPercentage;
	}
	public double getDeliveryCost() {
		return deliveryCost;
	}
	public void setDeliveryCost(double deliveryCost) {
		this.deliveryCost = deliveryCost;
	}
	
	
	public double getLastMonthIncome() {
		int theDay = new java.util.Date().getDay();
		int theMonth = new java.util.Date().getMonth() + 1 ;
		int theYear = new java.util.Date().getYear() + 1900;
		Others.Date today = new Others.Date(theYear, theMonth, theDay);
		int lastMonth = 0;
		int yearOfLastMonth = 0;
		if(theMonth == 1){
			yearOfLastMonth=today.getIntYear()-1;
			lastMonth = 12;
		}
		else{
			yearOfLastMonth=today.getIntYear();
			lastMonth=today.getIntMonth()-1;
		}
		double totalIncome = 0;
		for(Order order : this.globalHistoric.getListOrder()){
			if((order.getDate().getIntMonth()== lastMonth)&(order.getDate().getIntYear()==yearOfLastMonth)){
				totalIncome = totalIncome + order.getPrice();
			}
		}
		
		return totalIncome;
	}
	
	public ArrayList<Order> getOrderPendingList() {
		// TODO Auto-generated method stub
		return this.listPendingOrder;
	}

	public Order getPendingOrderByName(String string) {
		for(Order order : listPendingOrder){
			if(order.getNameOrder().equals(string))
				return order;
		}
		return null;
	}
	public Customer getCustomerByOrder(Order o){
		for(Customer c : this.getListCustomer().getList()){
			Order oc = c.getOrderByName(o.getNameOrder());
			if(oc == o)
				return c;
		}
		return null;
	}
	
	public int getNumberOfOrderLastMonth() {
		/*
		Date date = new Date(); // your date
	    Calendar cal = Calendar.getInstance();
	    cal.setTimeZone(TimeZone.getDefault());
	    cal.setTime(date);
	    int theYear = cal.get(Calendar.YEAR) + 1;
	    int theMonth = cal.get(Calendar.MONTH) + 2;
	    int theDay = cal.get(Calendar.DAY_OF_MONTH);
	    */
		int theDay = new java.util.Date().getDay();
		int theMonth = new java.util.Date().getMonth() + 1 ;
		int theYear = new java.util.Date().getYear() + 1900;
		Others.Date today = new Others.Date(theYear, theMonth, theDay);
		int lastMonth = 0;
		int yearOfLastMonth = 0;
		if(theMonth == 1){
			yearOfLastMonth=today.getIntYear()-1;
			lastMonth = 12;
		}
		else{
			yearOfLastMonth=today.getIntYear();
			lastMonth=today.getIntMonth()-1;
		}
		int nbOfOrder = 0;
		for(Order order : this.globalHistoric.getListOrder()){
			if((order.getDate().getIntMonth()== lastMonth)&&(order.getDate().getIntYear()==yearOfLastMonth)){
				nbOfOrder++;
			}
		}
		return nbOfOrder;
	}
	
	public double getGlobalProfit(){
		
		double totalProfit = 0;
		for(Order order : this.globalHistoric.getListOrder()){
			if(order != null){
				totalProfit += order.getPrice()*this.markupPercentage + this.serviceFee - this.deliveryCost ;
			}
		}
		return totalProfit;
	}
	
public double getProfitBetween(Others.Date dateBeg, Others.Date dateEnd){
		
		double totalProfit = 0;
		for(Order order : this.globalHistoric.getListOrder()){
			//System.out.println(dateBeg + " - " + order.getDate() + " - " + dateBeg.compareTo(order.getDate()));
			if(dateBeg.compareTo(order.getDate()) <= 0 && dateEnd.compareTo(order.getDate()) >= 0){
				totalProfit += order.getPrice()*this.markupPercentage + this.serviceFee - this.deliveryCost ;
			}
		}
		
		return totalProfit;
	}

}
