package Item;

import java.util.ArrayList;

/**
 * This Meal class is created by the use of a pattern given in the constructor of
 * subClasses, this pattern must consist on an array, each case contain an obligatory
 * typeOfMeal (starter,mainDish...), if the user have the choice between two typeOfMeal
 * it can be implemented by using a subArray.
 * For exemple, the patterns below show how a full meal may be created and then a half meal :
 * fullMealPattern={{"Starter"},{"MainDish"},{"Dessert"}}
 * halfMealPattern={{"Starter","Dessert"},{"MainDish"}}
 * 
 * We may create a children meal very easily :
 * chlidrenMealPattern={{"MainDish"},{"Dessert"},{"Drink"}}
 * 
 * @author Nicolas FLEY et Corentin RAVOUX
 *
 */

public abstract class Meal extends Item{
	
	ArrayList<Dish> listDish = null;
	String [][] patternTypeInMeal;
	protected String typeOfMeal;
	protected long nbOfSell;
	
	private double price=0;
	
	/** CONSTRUCTORS **/
	
	public Meal(String name,String typeOfFood) {
		super(name,typeOfFood);
		this.nbOfSell = 0;
		this.listDish = new ArrayList<Dish>();
	}
	
	/** add and remove dishes **/
	
	public void addDish(Dish dish){
		int idInMenu=-1;
		int idInMenuInc=0;
		for(String[] tabOfType : this.getPatternTypeInMeal()){
			for(String typeFromPatternType : tabOfType){
				if(typeFromPatternType.toLowerCase().equals(dish.getTypeInMeal().toLowerCase())){
					idInMenu=idInMenuInc;
				}
			}
			idInMenuInc+=1;
		}
		if(idInMenu>-1){
			this.listDish.set(idInMenu, dish);
		}
	}
	
	public void removeDish(String name){
		boolean itemRemoved = false;
		Dish dishToRemove = null;
		for(Dish dish: listDish){
			if(dish.getName()==name){
				dishToRemove = dish;
				itemRemoved=true;
			}
		}
		if(dishToRemove != null){
			listDish.remove(dishToRemove);
		}
		if(itemRemoved)
			System.out.println("Dish removed !");
	}
	public void removeDish(int num){
		Dish dish = listDish.get(num);
		if(dish.getName()==name)
			if(!listDish.contains(dish))
				listDish.add(dish);
			else{
				listDish.add(dish);
			}
	}
	
	/** GETTER AND SETTER **/
	
	
	
	public String getTypeOfMeal() {
		return typeOfMeal;
	}
	public long getNbOfSell() {
		return nbOfSell;
	}

	public void setNbOfSell(long nbOfSell) {
		this.nbOfSell = nbOfSell;
	}

	protected void setTypeOfMeal(String _typeOfMeal) {
		this.typeOfMeal = _typeOfMeal;
	}
	public double getPrice(){
		price = 0;
		for(Dish dish : listDish){
			if(dish!=null)
				price+=dish.getPrice();
		}
		return price;
	}
	public ArrayList<Dish> getListDish() {
		return listDish;
	}
	public String[][] getPatternTypeInMeal() {
		return patternTypeInMeal;
	}
	protected void setPatternTypeInMeal(String[][] patternTypeInMeal) {
		this.patternTypeInMeal = patternTypeInMeal;
	}

	
	/** OVERRIDE **/
	@Override
	public boolean equals(Object m){
		if(m instanceof Meal){
			Meal meal = (Meal) m;
			if(meal.getTypeOfMeal()==typeOfMeal)
				return true;
		}
		return false;
	}
	@Override
	public String toString(){
		String s="";
		s+="\n***\nMenu : "+name;
		s+=", this Menu contain : ";
		for(Dish d:listDish){
			if(d != null)
				s+=d.toString();
		}
		return s;
	}
}
