package Offers;

import java.util.ArrayList;

import Item.Meal;
import User.Restaurant;

/**
 * This class is attached to a unique meal for a restaurant.
 * It's also attached to a list of customers, if a meal is attached,
 * when the user log in, he/she receive an offer from this class.
 * 
 * @author Nicolas FLEY et Corentin RAVOUX
 *
 */

public class MealOfTheWeek implements ObservableOffer {
	
	
	private boolean state;
	private Meal meal;
	private boolean changed;
	private ArrayList<Observer> observers = new ArrayList<Observer>();
	
	
	
	
	
	public MealOfTheWeek(Meal meal) {
		super();
		this.state = true;
		this.meal = meal;
		this.changed = false;
		this.observers = new ArrayList<Observer>();
	}
	
	

	public boolean isState() {
		return state;
	}



	public void setState(boolean state) {
		this.state = state;
	}



	public Meal getMeal() {
		return meal;
	}



	public void setMeal(Meal meal) {
		this.meal = meal;
	}



	@Override
	public void attachObserver(Observer o) {
		this.observers.add(o);

	}

	@Override
	public void detachObserver(Observer o) {
		this.observers.remove(o);

	}
	
	

	public ArrayList<Observer> getObservers() {
		return observers;
	}



	public void setObservers(ArrayList<Observer> observers) {
		this.observers = observers;
	}

	@Override
	public void notifyObservers(Restaurant r) {
		if(this.changed){
			for (Observer observer : this.observers) {
				String message = "A new Offer Meal Of The Week is Available in the Restaurant : " + r.getName() + "\n "
						+ "Meal : " + this.meal.toString();
				observer.update(message);
			}
		this.changed=false;
		}
	}
		
	public void setInterests(Restaurant r){
		this.changed=true;
		this.notifyObservers(r);
	}

}


