package Order;

import java.util.ArrayList;
import Core.MyFoodora;
import Item.Dish;
import Item.Item;
import Item.Meal;
import Offers.MealOfTheWeek;
import Others.Period;
import User.Restaurant;

/**
 * A customer have has many order he wants to, each order is attached to
 * a restaurant and have numbers of his meals and dishes added by the customer.
 * When the order is finalized, price is calculated according to the strategyProfit.
 * Orders are also archived.
 * A Courier is selected to transmit the order to the customer.
 * 
 * @author Nicolas FLEY et Corentin RAVOUX
 *
 */

public class Order implements Comparable<Order>{
	
	private double price;
	private Others.Date date;
	private boolean finalized;
	private String nameOrder;
	private Restaurant restaurantAttached;

	public String getNameOrder() {
		return nameOrder;
	}

	ArrayList<Dish> listDish;
	ArrayList<Meal> listMeal;
	
	public Order() {
		super();
		
		this.listDish = new ArrayList<Dish>();
		this.listMeal = new ArrayList<Meal>();
	}
	
	public Order(String _name, Restaurant _restaurant) {
		super();
		
		this.listDish = new ArrayList<Dish>();
		this.listMeal = new ArrayList<Meal>();
		this.nameOrder = _name;
		this.restaurantAttached = _restaurant;
	}

	/** CUSTOM **/
	
	@Override
	public void finalize(){
		int theDay = new java.util.Date().getDay();
		int theMonth = new java.util.Date().getMonth() + 1 ;
		int theYear = new java.util.Date().getYear() + 1900;
		this.date = new Others.Date(theYear, theMonth, theDay);
		finalized = true;
	}
	
	public void finalize(MyFoodora foodora){
		int theDay = new java.util.Date().getDay();
		int theMonth = new java.util.Date().getMonth() + 1 ;
		int theYear = new java.util.Date().getYear() + 1900;
		this.date = new Others.Date(theYear, theMonth, theDay);
		finalized = true;
		foodora.getOrderPendingList().add(this);
	}
	
	public void finalize(MyFoodora foodora, Others.Date _date){
		this.date = _date;
		finalized = true;
		foodora.getOrderPendingList().add(this);
	}
	
	public static ArrayList<Order> getListOfOrderInAPeriod(HistoricOrder historic,Period period){
		ArrayList<Order> listOrder = new ArrayList<Order>();
		int beginDay = period.getBeginingOfPeriod().getIntDay();
		int endDay = period.getEndOfPeriod().getIntDay();
		int beginMonth = period.getBeginingOfPeriod().getIntMonth();
		int endMonth = period.getEndOfPeriod().getIntMonth();
		int beginYear = period.getBeginingOfPeriod().getIntYear();
		int endYear = period.getEndOfPeriod().getIntYear();
		for (Order order : historic.getListOrder()){
			Others.Date dateOfOrder = order.getDate();
			int dayOfOrder = dateOfOrder.getIntDay();
			int monthOfOrder = dateOfOrder.getIntMonth();
			int yearOfOrder = dateOfOrder.getIntYear();
			if((beginYear < yearOfOrder)  & ( yearOfOrder < endYear)){
				listOrder.add(order);
			}

			else if ((endYear == yearOfOrder)&(beginYear == yearOfOrder)){
				if((beginMonth < monthOfOrder) & (monthOfOrder < endMonth)){
					listOrder.add(order);
				}
				else if((beginMonth == monthOfOrder) & (endMonth == monthOfOrder)){
					if((beginDay< dayOfOrder)&(dayOfOrder<endDay)){
						listOrder.add(order);
					}
					else if ((endDay >= dayOfOrder) &(beginDay <= dayOfOrder)){
						listOrder.add(order);
					}
				}
				else if(beginMonth == monthOfOrder){
					if(beginDay <= dayOfOrder){
						listOrder.add(order);
					}
				}
				else if(endMonth == monthOfOrder){
					if(endDay >= dayOfOrder){
						listOrder.add(order);
					}
				}
			}
			else if (beginYear == yearOfOrder){
				if(beginMonth == monthOfOrder){
					if (beginDay <= dayOfOrder){
						listOrder.add(order);
					}
				}
				else if(beginMonth < monthOfOrder){
					listOrder.add(order);
				}

				
			}
			else if(endYear == yearOfOrder){
				if(endMonth == monthOfOrder){
					if(endDay >= dayOfOrder){
						listOrder.add(order);
					}
				if (endMonth > monthOfOrder){
					listOrder.add(order);
				}
				}
			}
			
		}
		
		return listOrder;
	}

	/** ADD & REMOVE **/
	
	public void addItemByName(String string) {
		Item item = this.restaurantAttached.getDishByName(string);
		if(item != null)
			this.add((Dish) item);
		else{
			item = this.restaurantAttached.getMealByName(string);
			if(item != null)
				this.add((Meal) item);
			else
				System.out.println("Unknown item.");
		}
	}
	
	public void add(Dish d){
		listDish.add(d);
		//price = price + d.getPrice();
	}
	public void add(Meal m){
		listMeal.add(m);
		//price = price + m.getPrice();
	}
	public void remove(Dish d){
		listDish.remove(d);
		//price = price - d.getPrice();
	}
	public void remove(Meal m){
		listMeal.remove(m);
		//price = price - m.getPrice();
	}
	public void removeDishById(int d){
		listDish.remove(d);
	}
	public void removeMealById(int m){
		listMeal.remove(m);
	}
	
	/** GETTER AND SETTER **/
	
	
	
	public boolean isFinalised(){
		return finalized;
	}
	public Restaurant getRestaurantAttached() {
		return restaurantAttached;
	}

	public void setRestaurantAttached(Restaurant restaurantAttached) {
		this.restaurantAttached = restaurantAttached;
	}

	public double getPrice(){
		price = 0;
		MealOfTheWeek mealOfTheWeekAttachedToRestaurant = restaurantAttached.getMealOfTheWeek();
		String nameOfTheMealOfTheWeek = "";
		if(mealOfTheWeekAttachedToRestaurant != null){
			if(mealOfTheWeekAttachedToRestaurant.getMeal() != null){
				for(Meal m : restaurantAttached.getListOfMeal()){
					if(mealOfTheWeekAttachedToRestaurant.getMeal().getName().toLowerCase().equals(m.getName().toLowerCase())){
						nameOfTheMealOfTheWeek = m.getName();
					}
				}
			}
		}
		for(Meal meal : listMeal){
			if(meal.getName().toLowerCase().equals(nameOfTheMealOfTheWeek.toLowerCase())){
				price += meal.getPrice()*(1-restaurantAttached.getSpecialDiscountFactor());
			}
			else{
				price += meal.getPrice()*(1-restaurantAttached.getGeneralDiscountFactor());
			}
		}
		for(Dish dish : listDish){
			price += dish.getPrice();
		}
		return price;
	}
	public void setPrice(double price) {
		this.price = price;
	}
	public Others.Date getDate(){
		return date;
	}
	
	
	public ArrayList<Dish> getListDish() {
		return listDish;
	}

	public void setListDish(ArrayList<Dish> listDish) {
		this.listDish = listDish;
	}

	public ArrayList<Meal> getListMeal() {
		return listMeal;
	}

	public void setListMeal(ArrayList<Meal> listMeal) {
		this.listMeal = listMeal;
	}
	
	@Override
	public String toString(){
		return "\n***\n Order, name : "+nameOrder+", date : "+ date +"\n Dishs : "+ listDish + "\n Meals : " + listMeal;
	}

	public void setDate(Others.Date date) {
		this.date = date;
	}

	@Override
	public int compareTo(Order o) {
		return 0;
	}
	
}
