package StrategyProfit;

import Core.ContextStrategy;
import Core.MyFoodora;
import Exception.ExceptionUnknownStartegyType;

/**
 * Context of the profit strategy. Allow the strategy to be saved and executed.
 * In fact, strategies are used to modify variable of foodora used to calculated
 * the order price.
 * 
 * @author Nicolas FLEY et Corentin RAVOUX
 *
 */

public class ContextTargetProfitStrategy implements ContextStrategy {
	
	public StrategyTargetProfitPolicy strategy;

	public ContextTargetProfitStrategy(String strategyName) throws ExceptionUnknownStartegyType {
		setStrategy(strategyName);
	}
	
	@Override
	public void setStrategy(String strategyName) throws ExceptionUnknownStartegyType{
		strategyName = strategyName.toLowerCase().trim();
		switch(strategyName){
		case "deliverycost":
			strategy = new TargetProfitDeliveryCost();
			break;
		case "servicefee":
			strategy = new TargetProfitServiceFee();
			break;
		case "markup":
			strategy = new TargetProfitMarkup();
			break;
		default:
			throw new ExceptionUnknownStartegyType("Unknown startegy '"+strategyName+"', DeliveryCost, ServiceFee and Markup available.");
		}
	}
	
	public void execute(MyFoodora myFoodora, double targetProfit){
		strategy.chooseTargetProfit(myFoodora, targetProfit);
	}

	@Override
	public String toString() {
		return strategy.toString();
	}

	public StrategyTargetProfitPolicy getStrategy() {
		return strategy;
	}

	public void setStrategy(StrategyTargetProfitPolicy strategy) {
		this.strategy = strategy;
	}
	
	
	
	

}
