package User;

import java.util.ArrayList;
import Cards.BasicFidelityCard;
import Cards.FidelityCard;
import Cards.LotteryFidelityCard;
import Cards.PointFidelityCard;
import Cards.VisiterCard;
import Others.Adress;
import Others.IDCard;
import Order.HistoricOrder;
import Order.Order;
import Core.MyFoodora;
import Item.Dish;
import Item.Meal;
import Offers.Observer;

/**
 * Customer may create order, add meals and dishes to it from a restaurant and ask to be delivered.
 * 
 * @author Nicolas FLEY et Corentin RAVOUX
 *
 */

public class Customer extends HumanUser implements VisiterCard,Observer {
	
	public static final int UNKNOWN = 0;
	public static final int COURIER = 1;
	public static final int RESTAURANT = 2;
	public static final int CUSTOMER = 3;
	public static final int MANAGER = 4;

	
	private boolean spamAgree;
	private FidelityCard card;
	private IDCard idCard;
	private ArrayList<Order> listOrder = new ArrayList<Order>();
	private ArrayList<String> listOffer = new ArrayList<String>();
	private long nbOfOfferReceived;
	
	public ArrayList<Order> getListOrder() {
		return listOrder;
	}

	public Customer(long phoneNumber, String name, String username, String mail, String password,
			Adress adress, boolean activated, String birthdayDate, String surname) {
		super(phoneNumber, name, username, mail, password, adress, activated, birthdayDate, surname);
		
		this.spamAgree = true;
		this.card = new BasicFidelityCard();
		this.typeOfUser = CUSTOMER;
		this.nbOfOfferReceived = 0;
	}
	
	public FidelityCard getCard() {
		return card;
	}


	public boolean isSpamAgree() {
		return spamAgree;
	}

	public void setSpamAgree(boolean spamAgree) {
		this.spamAgree = spamAgree;
	}

	@Override
	public String toString() {
		return "\n***\n" + this.name + ", mail : " + this.mail + ", phone : " + Long.toString(this.phoneNumber) + ", position : " + this.adress +
		", spamAgree : " + spamAgree + "\nhave been delivered : " + historic.getListOrder().size() + " times.\ncard : " + card.toString();
	}
	
	
	public void placeOrder(Restaurant restaurant, String mealOrDish, ArrayList<String> listOfItem, MyFoodora foodora){
		ArrayList<Dish> cartOfRestaurant = restaurant.getListOfDish();
		ArrayList<Meal> cartOfMeal = restaurant.getListOfMeal();
		Order order = new Order();
		order.setRestaurantAttached(restaurant);
		if(mealOrDish == "Dish"){
			for(String item : listOfItem){
				boolean isIn = false;
				Dish dishToChoose = null;
				for(Dish dish : cartOfRestaurant){
					if(dish.getName() == item){
						isIn = true;
						dishToChoose = dish;
						dish.setNbOfSell(dish.getNbOfSell() + 1);
					}
				if(isIn = false){
					System.out.println("This dish is not in the cart");
				}
				else{
					order.add(dishToChoose);
				}
				}
			}
		}
		else if(mealOrDish == "Meal"){
			for(String item : listOfItem){
				boolean isIn = false;
				Meal mealToChoose = null;
				for(Meal meal : cartOfMeal){
					if(meal.getName() == item){
						isIn = true;
						mealToChoose = meal;
						meal.setNbOfSell(meal.getNbOfSell() + 1);
					}
				}
				if(isIn = false){
					System.out.println("This meal is not in the menu");
				}
				else{
					order.add(mealToChoose);
				}
			}
		}
		this.historic.addOrder(order);
		restaurant.historic.addOrder(order);
		Courier chosenCourier = foodora.getContextDeliveryPolicy().getStrategy().chooseCourier(restaurant, this,foodora.getListCourier().getList());
		chosenCourier.acceptDelivery(order);
		foodora.getOrderPendingList().remove(order);
		foodora.getGlobalHistoric().addOrder(order);
		if (this.card instanceof PointFidelityCard){
			((PointFidelityCard) this.card).setNbOfPoint(this.getPointInTheCard() + 1);
		}
		double price = order.getPrice()*(1 + foodora.getMarkupPercentage()) + foodora.getDeliveryCost() + foodora.getServiceFee();
		boolean chance = false;
		if(this.card instanceof LotteryFidelityCard){
			chance = ((LotteryFidelityCard) this.card).getReduction();
		}
		if(chance){
			price = price*0.75;
		}
		System.out.println("You have to pay " + price);
	}
	
	public void placeOrder(Order order, MyFoodora foodora){
		Restaurant restaurant = order.getRestaurantAttached();
		for(Dish dish : order.getListDish()){
			dish.setNbOfSell(dish.getNbOfSell() + 1);
		}
		for(Meal meal : order.getListMeal()){
			meal.setNbOfSell(meal.getNbOfSell() + 1);
		}
		this.historic.addOrder(order);
		restaurant.historic.addOrder(order);
		Courier chosenCourier = foodora.getContextDeliveryPolicy().getStrategy().chooseCourier(restaurant, this,foodora.getListCourier().getList());
		foodora.getOrderPendingList().add(order);
		
		// should be a waiting time here
		if(chosenCourier == null)
			System.out.println("No courier available now");
		else{
			chosenCourier.acceptDelivery(order);
			foodora.getOrderPendingList().remove(order);
			foodora.getGlobalHistoric().addOrder(order);
			this.getListOrder().remove(order);
			double price = order.getPrice()*(1 + foodora.getMarkupPercentage()) + foodora.getDeliveryCost() + foodora.getServiceFee();
			if (this.card instanceof PointFidelityCard){
				PointFidelityCard fidelityCard = (PointFidelityCard) this.card;
				int numberOfPointForDiscount = fidelityCard.getNumberOfPointForDiscount();
				if(fidelityCard.getNbOfPoint()>numberOfPointForDiscount){ // result of the last order
					System.out.println("You got a 10% discount on the final price !");
					price*=0.9;
					fidelityCard.setNbOfPoint(this.getPointInTheCard() - 10);
				}
				fidelityCard.setNbOfPoint(this.getPointInTheCard() + ((int) price/10));
				if(fidelityCard.getNbOfPoint()>numberOfPointForDiscount)
					System.out.println("You reached "+numberOfPointForDiscount+" points, next times you'll get a 10% discount !");
			}
			boolean chance = false;
			if(this.card instanceof LotteryFidelityCard){
				chance = ((LotteryFidelityCard) this.card).getReduction();
			}
			if(chance){
				System.out.println("**********************\n"
								 + "** Congratulation ! **\n"
								 + "**********************\n"
								 + "You won a 25% discount on the current command order.");
				price = price*0.75;
			}
			System.out.println("You have to pay " + price);
		}
	}
	
	public void placeOrder(Order order, MyFoodora foodora, Others.Date date){
		Restaurant restaurant = order.getRestaurantAttached();
		order.setDate(date);
		if(!foodora.getOrderPendingList().contains(order)){
			for(Dish dish : order.getListDish()){
				dish.setNbOfSell(dish.getNbOfSell() + 1);
			}
			for(Meal meal : order.getListMeal()){
				meal.setNbOfSell(meal.getNbOfSell() + 1);
			}
			this.historic.addOrder(order);
			restaurant.historic.addOrder(order);
			foodora.getOrderPendingList().add(order);
		}
		Courier chosenCourier = foodora.getContextDeliveryPolicy().getStrategy().chooseCourier(restaurant, this,foodora.getListCourier().getList());
		
		// should be a waiting time here
		if(chosenCourier == null)
			System.out.println("No courier available now");
		else{
			chosenCourier.acceptDelivery(order);
			order.setDate(date);
			foodora.getOrderPendingList().remove(order);
			foodora.getGlobalHistoric().addOrder(order);
			if (this.card instanceof PointFidelityCard){
				((PointFidelityCard) this.card).setNbOfPoint(this.getPointInTheCard() + 1);
			}
			double price = order.getPrice()*(1 + foodora.getMarkupPercentage()) + foodora.getDeliveryCost() + foodora.getServiceFee();
			boolean chance = false;
			if(this.card instanceof LotteryFidelityCard){
				chance = ((LotteryFidelityCard) this.card).getReduction();
			}
			if(chance){
				price = price*0.75;
			}
			System.out.println("You have to pay " + price);
			System.out.println("Your delivrer is " + chosenCourier.getName());
		}
	}
	
	/*
	
	public void placeOrder(Restaurant restaurant){
		Order order = new Order();
		System.out.println(restaurant.toString());
		do {
			if (Confirm.text("Do you want a Meal?")){
				System.out.println("listdesMeal"); //  faire
				String choice = Cli.Input.string("Which meal do you want?");
				boolean isIn = false;
				for(Meal meal : restaurant.listOfMeal){
					if(meal.toString() == choice){
						isIn = true;
					}
				}
				if (isIn == false){
					System.out.println("This meal isn't propose by this restaurant");
				}
				else {
					if(choice == "FullMeal"){
						System.out.println("listedesDish"); // faire
						String entry1 = Cli.Input.string("Which entry do you want in your meal?");
						String main1 = Cli.Input.string("Which main do you want in your meal?");
						String dessert1 = Cli.Input.string("Which dessert do you want in your meal?");
						MealFactory factory1 = new MealFactory();
						Meal newmeal = factory1.createMeal("fullmeal", "meal1", "meal1");
					}
					if (choice == "HalfMeal"){
						System.out.println("listedesDish"); // faire
						String typeOfFormule = Cli.Input.string("Which formula do you want? Entry/Main(EM) or Main/Dessert(MD)");
						if(typeOfFormule == "EM"){
							String entry2 = Cli.Input.string("Which entry do you want in your meal?");
							String main2 = Cli.Input.string("Which main do you want in your meal?");
							MealFactory factory2 = new MealFactory();
							//Meal newmeal = factory2.createMeal("halfmeal"
						}
						else if(typeOfFormule == "MD"){
							String main3 = Cli.Input.string("Which main do you want in your meal?");
							String dessert3 = Cli.Input.string("Which dessert do you want in your meal?");
							MealFactory factory3 = new MealFactory();
							//Meal newmeal = factory3.createMeal("halfmeal"
						}
						else{
							System.out.println("This type of Meal is not available");
						}
					}

					}
				
				
			}
			else if (Confirm.text("Do you want a dish?")){
				System.out.println("listOfDish");
				String dish = Cli.Input.string("Which dish do you want ?");
				FactoryDish factory4 = new FactoryDish();
				
		
				
			}
			
			
		} while (Confirm.text("Do you want something else ?"));
		System.out.println("Do you want a Meal? (y)es or (n)o");
		Scanner sc = new Scanner(System.in);
		String choice = sc.nextLine();
		if (Confirm.text("Do you want a Meal?")){
			
		}
		
	} */
	
	public void registerFidelityPlan(String newTypeOfFidelityCard){
		if(newTypeOfFidelityCard == this.card.returnType()){
			System.out.println("You already have this card");}
		else if (!
				(newTypeOfFidelityCard.toLowerCase().equals("basic")
				|| newTypeOfFidelityCard.toLowerCase().equals("lottery")
				|| newTypeOfFidelityCard.toLowerCase().equals("point"))
				){
			System.out.println("You have to choose between Point, Lottery card");
		}
		else if (newTypeOfFidelityCard.toLowerCase().equals("basic")){
			System.out.println("You have to choose between Point, Lottery card");
		}
		else {
			if (newTypeOfFidelityCard.toLowerCase().equals("point")){
				idCard=IDCard.getInstance();
				this.card = new PointFidelityCard();
			}
			
			if (newTypeOfFidelityCard.toLowerCase().equals("lottery")){
				idCard=IDCard.getInstance();
				this.card = new LotteryFidelityCard();
			}
			System.out.println("Successfully done");
		}
	}
	
	public void unregisterFidelityPlan(){
		if(this.card.returnType() == "Basic"){
			System.out.println("You already have this card");}
	
		else {
			idCard=IDCard.getInstance();
			this.card = new BasicFidelityCard();
		}
	}
	
	public HistoricOrder getHistorique() {
		return historic;
	}
	
	public long getPointInTheCard(){
		if((this.card.returnType() == "Basic") || (this.card.returnType() == "Lottery")){
			System.out.println("You don't have the right type of Card");
			return 0;
		}
		else {
			return visit((PointFidelityCard)card);
		}
	}
	
	public double getPercentageInTheCard(){
		if((this.card.returnType() == "Point") || (this.card.returnType() == "Basic")){
			System.out.println("You don't have the right type of Card");
			return 0;
		}
		else {
			return visit((LotteryFidelityCard)card);
		}
	}
	
	@Override
	public double visit(LotteryFidelityCard card){
		return card.acceptPercentage();
	}

	@Override
	public long visit(PointFidelityCard card) {
		return card.acceptPoint();
	}
	
	public void acceptSpam (){
		if (spamAgree == true){
			System.out.println("You've already accept to receive notifications");
		}
		else {
			this.spamAgree = true;
			
		}
	}
	
	public void refuseSpam(){
		if (spamAgree == false){
			System.out.println("You've already refuse to receive notifications");
		}
		else {
			this.spamAgree = false;
		}
	}



	public void createOrder(String name, Restaurant restaurant){
		Order order = new Order(name,restaurant);
		listOrder.add(order);
	}
	
	public Order getOrderByName(String string) {
		for(Order order : listOrder){
			if(order.getNameOrder().toLowerCase().equals(string.toLowerCase()))
				return order;
		}
		System.out.println(this.getListOrder());
		System.out.println("\n/!\\ WARNING : " + string + " order is unkown.");
		return null;
	}

	public void checkForPendingOffer(){
		if(listOffer.size() > 0){
			for(String offer : listOffer){
				System.out.println("\n/!\\ An offer has been received /!\\\n" + offer);
			}
			listOffer.clear();
		}
	}
	
	@Override
	public void update(String offer) {
		listOffer.add(offer);
		this.nbOfOfferReceived++;
	}

	public long getNbOfOfferReceived() {
		return nbOfOfferReceived;
	}
	
	
	
}
	
	


