package User;
import Core.MyFoodora;
import DeliveryStrategy.ContextDeliveryStrategy;
import Exception.ExceptionUnknownTypeOfUser;
import Item.Dish;
import Item.HalfMeal;
import Order.HistoricOrder;
import Order.Order;

import java.util.ArrayList;
import Cards.FidelityCard;
import Cards.LotteryFidelityCard;
import Others.Adress;
import StrategyProfit.ContextTargetProfitStrategy;
import UserFactoryPattern.FactoryUserFactory;
import UserFactoryPattern.UserFactory;

/**
 * The managers controls foodora, they are able to do anything in the system
 * Creating users, changing parameters, calculating statistics.
 * This class contain a lot of methods describing theses abilities.
 * 
 * @author Nicolas FLEY et Corentin RAVOUX
 *
 */

public class Manager extends HumanUser {
	
	public static final int UNKNOWN = 0;
	public static final int COURIER = 1;
	public static final int RESTAURANT = 2;
	public static final int CUSTOMER = 3;
	public static final int MANAGER = 4;
	

	public Manager(long phoneNumber, String name, String username, String mail, String password, Adress adress,
			boolean activated, String birthdayDate, String surname) {
		super(phoneNumber, name, username, mail, password, adress, activated, birthdayDate, surname);
		this.typeOfUser = MANAGER;
	}
	

	
	public void addUser(long phoneNumber, String name, String username, String mail, String password, Adress adress,
			boolean activated , String birthdayDate, String surname,MyFoodora foodora, int typeOfUser, String kindOfUser) throws ExceptionUnknownTypeOfUser{
		
		if ((kindOfUser == "Human") || (kindOfUser == "Moral")){
			UserFactory factory = FactoryUserFactory.createUserFactory(kindOfUser);
			if((kindOfUser == "Human")){
				HumanUser humanUser = factory.createHumanUser(phoneNumber, name, username, mail, password, adress, activated, birthdayDate, surname, typeOfUser);
				if(humanUser != null){
					foodora.addUser(humanUser);
					if(typeOfUser == CUSTOMER){
						foodora.addCustomer((Customer) humanUser);
					}
					else if (typeOfUser == MANAGER){
						foodora.addManager((Manager) humanUser);
					}
					else if (typeOfUser == COURIER){
						foodora.addCourier((Courier) humanUser);
					}
				}
			}
			else if (kindOfUser == "Moral"){
				MoralUser moralUser = factory.createMoralUser(phoneNumber, name, username, mail, password, adress, activated, typeOfUser);
				if(moralUser != null){
					foodora.addUser(moralUser);
					if(typeOfUser == RESTAURANT){
						foodora.addRestaurant((Restaurant) moralUser);
					}
				}
			}
		
		}
		else {
			System.out.println("Please choose your kind of User between Human or Moral");
		}

		
	}
	
	public void removeUser(User u, MyFoodora foodora){
		long idUser = u.getId();
		int typeOfUser = u.getTypeOfUser();
		foodora.removeWithID(typeOfUser, idUser);
		}
	
	
	public void changeServiceFee(double newServiceFee, MyFoodora foodora){
		foodora.setServiceFee(newServiceFee);
	}
	
	public void changeMarkupPercentage (double newMarkupPercentage, MyFoodora foodora){
		foodora.setMarkupPercentage(newMarkupPercentage);
	}
	
	public void changeDeliveryCost(double newDeliveryCost, MyFoodora foodora){
		foodora.setDeliveryCost(newDeliveryCost);
	}
	
	public double getTotalIncome(Others.Period period, MyFoodora foodora){
		HistoricOrder historicOfFoodora = foodora.getGlobalHistoric();
		ArrayList<Order> listOrderOverPeriod = Order.getListOfOrderInAPeriod(historicOfFoodora,period);
		double totalIncome = 0;
		for(Order order : listOrderOverPeriod){
			totalIncome = totalIncome + order.getPrice();
		}
		return totalIncome;
	}
	
	public double getTotalProfit(Others.Period period, MyFoodora foodora){
		HistoricOrder historicOfFoodora = foodora.getGlobalHistoric();
		ArrayList<Order> listOrderOverPeriod = Order.getListOfOrderInAPeriod(historicOfFoodora,period);
		double totalIncome = 0;
		for(Order order : listOrderOverPeriod){
			totalIncome = totalIncome + order.getPrice()*foodora.getMarkupPercentage() + foodora.getServiceFee() - foodora.getDeliveryCost();
		}
		return totalIncome;
	}
	
	public double getAverageIncomePerCustomer(MyFoodora foodora, Others.Period period){
		ListUser<Customer> listOfCustomer = foodora.getListCustomer();
		double nbOfCustomer = 0;
		for (Customer customer : listOfCustomer.getList()){
			HistoricOrder historicOfCustomer = customer.getHistoricOrder();
			ArrayList<Order> listOrderOverPeriod = Order.getListOfOrderInAPeriod(historicOfCustomer, period);
			if(listOrderOverPeriod.size() >= 1){
				nbOfCustomer = nbOfCustomer + 1;
			}
		}
		double totalIncome = getTotalIncome(period, foodora);
		
		if(nbOfCustomer != 0){
			return totalIncome/nbOfCustomer;

		}
		else {
			return 0;
		}
	}
	
	
	public void setTargetProfitPolicy(MyFoodora foodora, ContextTargetProfitStrategy newStrategy){
		ContextTargetProfitStrategy oldStrategy = foodora.getContextTargetProfitStrategy();
		if(oldStrategy == newStrategy){
			System.out.println("This Strategy is already in place");
		}
		else{
			foodora.setContextTargetProfitStrategy(newStrategy);
		}
		
	}
	
	public void setDeliveryPolicy(MyFoodora foodora, ContextDeliveryStrategy newStrategy){
		ContextDeliveryStrategy oldStrategy = foodora.getContextDeliveryPolicy();
		if(oldStrategy == newStrategy){
			System.out.println("This Strategy is already in place");
		}
		else{
			foodora.setContextDeliveryPolicy(newStrategy);
		}
			
	}
	
	public void executeTargetProfitPolicy(MyFoodora foodora, double targetProfit){
		foodora.getContextTargetProfitStrategy().execute(foodora, targetProfit);
	}
	
	public String getMostSellingRestaurant(MyFoodora foodora){
		ListUser<Restaurant> listOfRestaurant = foodora.getListRestaurant();
		Restaurant mostSellingRestaurant = null;
		double totalIncomeMax = 0;
		for(Restaurant restaurant : listOfRestaurant.getList()){
			HistoricOrder listOfOrder = restaurant.getHistoricOrder();
			double totalIncome = 0;
			for(Order order : listOfOrder.getListOrder()){
				totalIncome = totalIncome + order.getPrice();
			}
			if(totalIncome >= totalIncomeMax){
				totalIncomeMax = totalIncome;
				mostSellingRestaurant = restaurant;
			}
		}
		return mostSellingRestaurant.getName();
	}
	
	public String getLeastSellingRestaurant(MyFoodora foodora){
		ListUser<Restaurant> listOfRestaurant = foodora.getListRestaurant();
		Restaurant leastSellingRestaurant = null;
		double totalIncomeMin = Math.pow(10, 10);
		for(Restaurant restaurant : listOfRestaurant.getList()){
			HistoricOrder listOfOrder = restaurant.getHistoricOrder();
			double totalIncome = 0;
			for(Order order : listOfOrder.getListOrder()){
				totalIncome = totalIncome + order.getPrice();
			}
			if(totalIncome <= totalIncomeMin){
				totalIncomeMin = totalIncome;
				leastSellingRestaurant = restaurant;
			}
		}
		return leastSellingRestaurant.nameOf();
	}
	
	public String getLeastActiveCourier(MyFoodora foodora){
		ListUser<Courier> listOfCourier = foodora.getListCourier();
		long leastNbOfSell = 100000000;
		Courier leastSeller = null;
		for (Courier courier : listOfCourier.getList()) {
			if(courier.getNbOfLivraison() <= leastNbOfSell){
				leastSeller = courier;
				leastNbOfSell = leastSeller.getNbOfLivraison();
			}
		}
		return leastSeller.getName() ;
		
	}
	
	public String getMostActiveCourier(MyFoodora foodora){
		ListUser<Courier> listOfCourier = foodora.getListCourier();
		long mostNbOfSell = 0;
		Courier mostSeller = null;
		for (Courier courier : listOfCourier.getList()) {
			if(courier.getNbOfLivraison() >= mostNbOfSell){
				mostSeller = courier;
				mostNbOfSell = mostSeller.getNbOfLivraison();
			}
		}
		return mostSeller.getName() ;
		
	}

	public void activateUser(MyFoodora foodora, User user){
		ListUser<User> listOfUser = foodora.getGlobalListUser();
		ListUser<Customer> listOfCustomer = foodora.getListCustomer();
		ListUser<Courier> listOfCourier = foodora.getListCourier();
		ListUser<Restaurant> listOfRestaurant = foodora.getListRestaurant();
		ListUser<Manager> listOfManager = foodora.getListManager();
		for(User u : listOfUser.getList()){
			if(u.equals(user)){
				if(u.isActivated() == true){
					System.out.println("This user is already activated");
					break;
				}
				else{
					User uActivate = u;
					listOfUser.setUseractivated(true, uActivate);
					foodora.setGlobalListUser(listOfUser);
					
					if(u.getTypeOfUser() == CUSTOMER){
						listOfCustomer.setUseractivated(true,(Customer) uActivate);
						foodora.setListCustomer(listOfCustomer);
					}
					if(u.getTypeOfUser() == RESTAURANT){
						listOfRestaurant.setUseractivated(true, (Restaurant) uActivate);
						foodora.setListRestaurant(listOfRestaurant);
					}
					if(u.getTypeOfUser() == COURIER){
						listOfCourier.setUseractivated(true, (Courier) uActivate);
						foodora.setListCourier(listOfCourier);
					}
					if(u.getTypeOfUser() == MANAGER){
						listOfManager.setUseractivated(true, (Manager) uActivate);
						foodora.setListManager(listOfManager);
					}
				
			}
			}

		}

	}
	
	public void desactivateUser(MyFoodora foodora, User user){
		ListUser<User> listOfUser = foodora.getGlobalListUser();
		ListUser<Customer> listOfCustomer = foodora.getListCustomer();
		ListUser<Courier> listOfCourier = foodora.getListCourier();
		ListUser<Restaurant> listOfRestaurant = foodora.getListRestaurant();
		ListUser<Manager> listOfManager = foodora.getListManager();
		for(User u : listOfUser.getList()){
			if(u.equals(user)){
				if(u.isActivated() == false){
					System.out.println("This user is already deactivated");
					break;
				}
				else{
					User uActivate = u;
					listOfUser.setUseractivated(false, uActivate);
					foodora.setGlobalListUser(listOfUser);
					
					if(u.getTypeOfUser() == CUSTOMER){
						listOfCustomer.setUseractivated(false,(Customer) uActivate);
						foodora.setListCustomer(listOfCustomer);
					}
					if(u.getTypeOfUser() == RESTAURANT){
						listOfRestaurant.setUseractivated(false, (Restaurant) uActivate);
						foodora.setListRestaurant(listOfRestaurant);
					}
					if(u.getTypeOfUser() == COURIER){
						listOfCourier.setUseractivated(false, (Courier) uActivate);
						foodora.setListCourier(listOfCourier);
					}
					if(u.getTypeOfUser() == MANAGER){
						listOfManager.setUseractivated(false, (Manager) uActivate);
						foodora.setListManager(listOfManager);
					}
				
			}
			}

		}

	}
	
	public void setProbabilityOnLoteryCard(FidelityCard c, double percentage){
		if(c instanceof LotteryFidelityCard){
			((LotteryFidelityCard) c).setPercentage(percentage);
		}
	}
	
	
	public ArrayList<HalfMeal> getSortedListOfHalfMeal (Restaurant restaurant){
		return restaurant.getSortListOfHalfMeal();
	}
	
	
	public ArrayList<Dish> getSortedListOfDish (Restaurant restaurant){
		return restaurant.getSortListOfDish();
	}
	
}
