package User;

import java.util.ArrayList;

import Core.MyFoodora;
import Exception.ExceptionUnknownDishType;
import Exception.ExceptionUnknownMealType;
import Item.Dish;
import Item.FactoryDish;
import Item.FactoryMeal;
import Item.HalfMeal;
import Item.Item;
import Item.Meal;
import Offers.MealOfTheWeek;
import Others.Adress;

/**
 * Restaurant are able to manage themself, they may create there meals, dishes,
 * set the mealOfTheWeek or specify discount factor.
 * 
 * @author Nicolas FLEY et Corentin RAVOUX
 *
 */

public class Restaurant extends MoralUser {
	
	private ArrayList<Meal> listOfMeal;
	private ArrayList<Dish> listOfDish;
	private double generalDiscountFactor;
	private double specialDiscountFactor;
	private MealOfTheWeek mealOfTheWeek;
	
	/** CONSTRUCTOR **/
	
	public Restaurant(long phoneNumber, String name, String username, String mail, String password,
			Adress adress, boolean activated) {
		super(phoneNumber, name, username, mail, password, adress, activated);
		this.listOfMeal = new ArrayList<Meal>();
		this.listOfDish = new ArrayList<Dish>();
		this.mealOfTheWeek = new MealOfTheWeek(null);
		this.typeOfUser = User.RESTAURANT;
		this.generalDiscountFactor = 0.05;
		this.specialDiscountFactor = 0.1;
	}


	/** SETTERS
	 * @throws ExceptionUnknownMealType **/
	public void addMeal(String nameOfMeal, String typeOfMeal, ArrayList<Dish> listOfDish) throws ExceptionUnknownMealType{
		FactoryMeal factory1 = new FactoryMeal();
		Meal m = factory1.createMeal(nameOfMeal, typeOfMeal);
		if(((listOfDish.size() !=2) &(typeOfMeal == "Half")) || ((listOfDish.size() !=3) &(typeOfMeal == "Full"))){
			System.out.println("Not the correct list of Dish");
		}
		if(typeOfMeal == "Half"){
			for(Dish dish : listOfDish){
				m.addDish(dish);
			}
		}
		boolean isalready = false;
		for (Meal meal : this.listOfMeal) {
			if(meal.equals(m)){
				System.out.println("You already have this meal in our menu");
				isalready = true;
			}
		}
		if(isalready == false){
			this.listOfMeal.add(m);
		}
	}
	
    public Meal createMeal(String nameOfMeal, String typeOfMeal) throws ExceptionUnknownMealType{
		FactoryMeal factory = new FactoryMeal();
		Meal m = factory.createMeal(nameOfMeal, typeOfMeal);
		listOfMeal.add(m);
		return m;
	}
	
	public void addDishToMeal(Dish d, Meal m){
		m.addDish(d);
	}
		
    public Dish getDishByName(String name){
		for(Dish d : listOfDish){
			if(d.getName().toLowerCase().equals(name.toLowerCase())){
				return d;
			}
		}
		return null;
	}
    
    public Meal getMealByName(String name){
		for(Meal m : listOfMeal){
			if(m.getName().toLowerCase().equals(name.toLowerCase())){
				return m;
			}
		}
		return null;
	}
    
    public Item getItemByName(String name){
		Item item = getMealByName(name);
		if(item == null)
			item = getDishByName(name);
		return item;
	}
	
	public String nameOf(){
		return this.getName();
	} 
	
	@Override
	public String toString() {
		return "\n***\nRestaurant " + this.name + ", mail : " + this.mail + ", tel : "+ this.phoneNumber +", position : " + this.adress
				+ "\nList of Dishs : " + this.getListOfDish()
				+ "\nList of Meals : " + this.getListOfMeal();
	}


	public ArrayList<Meal> getListOfMeal() {
		return listOfMeal;
	}


	public ArrayList<Dish> getListOfDish() {
		return listOfDish;
	}


	public double getGeneralDiscountFactor() {
		return generalDiscountFactor;
	}

	public void setGeneralDiscountFactor(double generalDiscountFactor) {
		this.generalDiscountFactor = generalDiscountFactor;
	}

	public double getSpecialDiscountFactor() {
		return specialDiscountFactor;
	}

	public void setSpecialDiscountFactor(double specialDiscountFactor) {
		this.specialDiscountFactor = specialDiscountFactor;
	}

	public void addDish(String typeInMeal, String name, String typeOfFood, double price) throws ExceptionUnknownDishType{
		FactoryDish factory1 = new FactoryDish();
		Dish d = factory1.createDish(typeInMeal, name, typeOfFood, price);
		boolean isalready = false;
		for (Dish dish : this.listOfDish) {
			if(dish.equals(d)){
				System.out.println("You already have this dish in our menu");
				isalready = true;
			}
		}
		if(isalready == false){
			this.listOfDish.add(d);

		}
	}
	
	
	public void removeMeal(Meal m){
		this.listOfMeal.remove(m);
		
	}
	
	public void removeDish(Dish d){
		this.listOfDish.remove(d);
	}

	public ArrayList<HalfMeal> getSortListOfHalfMeal(){
		ArrayList<HalfMeal> listOfHalMeal = new ArrayList<HalfMeal>();
		ArrayList<HalfMeal> sortedListOfHalMeal = new ArrayList<HalfMeal>();
		for(Meal m : this.listOfMeal){
			if(m.getTypeOfMeal() == "HalfMeal"){
				listOfHalMeal.add((HalfMeal) m);
			}
		}
		while(listOfHalMeal.isEmpty() == false){
			long maxNbOfSell = 0;
			HalfMeal maxSelledInTheList = null;
			for(HalfMeal m : listOfHalMeal){
				if(m.getNbOfSell() >= maxNbOfSell){
					maxNbOfSell = m.getNbOfSell();
					maxSelledInTheList = m;
				}
			}
			sortedListOfHalMeal.add(maxSelledInTheList);
			listOfHalMeal.remove(maxSelledInTheList);
			
		}
		return sortedListOfHalMeal;
	}
	
	
	public ArrayList<Dish> getSortListOfDish(){
		ArrayList<Dish> listToSort = new ArrayList<Dish>();
		ArrayList<Dish> sortedListOfDish = new ArrayList<Dish>();
		for(Dish dish : listOfDish){
			listToSort.add(dish);
		}
		while(listToSort.isEmpty() == false){
			long maxNbOfSell = 0;
			Dish maxSelledInTheList = null;
			for(Dish d : listToSort){
				if(d.getNbOfSell() >= maxNbOfSell){
					maxNbOfSell = d.getNbOfSell();
					maxSelledInTheList = d;
				}
			}
			sortedListOfDish.add(maxSelledInTheList);
			listToSort.remove(maxSelledInTheList);
			
		}
		return sortedListOfDish;
	}
	
	
	public void addMealOfTheWeek(MyFoodora foodora, String nameOfMeal){
		for(Meal meal : listOfMeal){
			if(meal.getName().toLowerCase().equals(nameOfMeal.toLowerCase())){
				this.mealOfTheWeek.setMeal(meal);
			}
		}
		for(Customer c : foodora.getListCustomer().getList()){
			if(c.isSpamAgree()){
				this.mealOfTheWeek.attachObserver(c);
			}
		}
		this.mealOfTheWeek.setInterests(this);
	}
	
	public void removeMealOfTheWeek(MyFoodora foodora){
		this.mealOfTheWeek = new MealOfTheWeek(null);
	}

	public MealOfTheWeek getMealOfTheWeek() {
		return mealOfTheWeek;
	}

	public void setMealOfTheWeek(MealOfTheWeek mealOfTheWeek) {
		this.mealOfTheWeek = mealOfTheWeek;
	}
	
	
	
}
