package User;

import Others.Adress;
import Order.HistoricOrder;

/**
 * Abstraction of all the available users.
 * Each available user have a list of itself stocked in foodora.
 * 
 * @author Nicolas FLEY et Corentin RAVOUX
 *
 */

public abstract class User implements Comparable<User> {
	
	protected long id;
	protected long phoneNumber;
	protected String name,username,mail,password;
	protected Adress adress;
	protected boolean activated;
	protected HistoricOrder historic;
	protected boolean isLogged;
	protected int typeOfUser;

	public static final int UNKNOWN = 0;
	public static final int COURIER = 1;
	public static final int RESTAURANT = 2;
	public static final int CUSTOMER = 3;
	public static final int MANAGER = 4;
	
	/** CONSTRUCTOR **/
	
	public User() {
	}
	
	public User(long phoneNumber, String name, String username, String mail, String password, Adress adress,
			boolean activated) {
		super();
		IDUser idUser = IDUser.getInstance();
		this.id = idUser.getNextSerialNumber();
		this.historic = new HistoricOrder();
		this.isLogged = false;
		
		this.phoneNumber = phoneNumber;
		this.name = name;
		this.username = username;
		this.mail = mail;
		this.password = password;
		this.adress = adress;
		this.activated = activated;
		
	}

	/** CUSTOM **/
	
	public boolean isLoginCorrect(String username, String password) {
		if(this.username.equals(username) && this.password.equals(password))
			return true;
		return false;
	}

	/** OVERRIDE **/
	
	@Override
	public int compareTo(User u){
		return username.compareTo(u.getUsername());
	}
	
	@Override
	public boolean equals(Object other){
		if(other instanceof User){
			User u = (User) other;
			if(this.id == u.getId()){
				return true;
			}
		}
		return false;
	}
	
	@Override
	public String toString() {
		return this.name + ", mail : " + this.mail + ", phone : " + Long.toString(this.phoneNumber) + ", position : " + this.adress;
	}
	
	/** GETTER & SETTER **/
	
	
	
	
	public long getId() {
		return id;
	}
	
	public int getTypeOfUser() {
		return typeOfUser;
	}

	public void setTypeOfUser(int typeOfUser) {
		this.typeOfUser = typeOfUser;
	}

	public HistoricOrder getHistoricOrder() {
		return historic;
	}
	
	public boolean isLogged(){
		return this.isLogged;
	}
	
	public long getPhoneNumber() {
		return phoneNumber;
	}
	public void setPhoneNumber(long phoneNumber) {
		this.phoneNumber = phoneNumber;
	}
	
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}
	
	public String getUsername() {
		return username;
	}
	public void setUsername(String username) {
		this.username = username;
	}
	
	public String getMail() {
		return mail;
	}
	public void setMail(String mail) {
		this.mail = mail;
	}
	
	public void setpassword(String password) {
		this.password = password;
	}
	
	public Adress getAdress() {
		return adress;
	}
	public void setAdress(Adress adress) {
		this.adress = adress;
	}
	
	public boolean isActivated() {
		return activated;
	}
	public void setActivated(boolean activated) {
		this.activated = activated;
	}
	
}
