package test;

import static org.junit.Assert.*;

import org.junit.Test;

import Core.MyFoodora;
import Exception.ExceptionUnknownStartegyType;
import Exception.ExceptionUnknownTypeOfUser;
import Item.Dish;
import Order.Order;
import Others.Adress;
import Others.Date;
import User.Customer;
import User.Manager;
import User.Restaurant;

public class TestCore {
	
	public static final int UNKNOWN = 0;
	public static final int COURIER = 1;
	public static final int RESTAURANT = 2;
	public static final int CUSTOMER = 3;
	public static final int MANAGER = 4;

	@Test
	public void testGetUserByLogin() throws ExceptionUnknownTypeOfUser, ExceptionUnknownStartegyType {
		MyFoodora foodora = new MyFoodora();
		Manager manager = new Manager(0674, "jacques", 	"jacquo", "jacques.martin@hotmail.fr", "123456789", new Adress(15.0, 10.0), true, "25/15/1955", "jacquo");
		manager.addUser(0145, "henry", "henry", "henry.dupont@hotmail.fr", "124578", new Adress(11.0, 15.2), true, "25/12/1994", "henrynou", foodora, CUSTOMER, "Human");
		Customer customer = (Customer) foodora.getUserByLogin("henry", "124578");
		assertTrue(customer.getName() == "henry");
	}
	
	@Test
	public void testGetUserByName() throws ExceptionUnknownTypeOfUser, ExceptionUnknownStartegyType {
		MyFoodora foodora = new MyFoodora();
		Manager manager = new Manager(0674, "jacques", 	"jacquo", "jacques.martin@hotmail.fr", "123456789", new Adress(15.0, 10.0), true, "25/15/1955", "jacquo");
		manager.addUser(0145, "henry", "henry", "henry.dupont@hotmail.fr", "124578", new Adress(11.0, 15.2), true, "25/12/1994", "henrynou", foodora, CUSTOMER, "Human");
		Customer customer = (Customer) foodora.getUserByName("henry", "henry");
		assertTrue(customer.getName() == "henry");
	}
	
	@Test
	public void testRemoveWithID() throws ExceptionUnknownTypeOfUser, ExceptionUnknownStartegyType{
		MyFoodora foodora = new MyFoodora();
		Manager manager = new Manager(0674, "jacques", 	"jacquo", "jacques.martin@hotmail.fr", "123456789", new Adress(15.0, 10.0), true, "25/15/1955", "jacquo");
		manager.addUser(0145, "henry", "henry", "henry.dupont@hotmail.fr", "124578", new Adress(11.0, 15.2), true, "25/12/1994", "henrynou", foodora, CUSTOMER, "Human");
		Customer henry = (Customer) foodora.getUserByName("henry", "henry");
		foodora.removeWithID(CUSTOMER, henry.getId());
		assertTrue(foodora.getListCustomer().getList().size() ==0);

	}
	
	@Test
	public void testGetLastMonthIncome() throws ExceptionUnknownStartegyType{
		MyFoodora foodora = new MyFoodora();
		Restaurant resto = new Restaurant(0245, "La tanire", "Lataniere", "contact@lataniere.fr", "1234", new Adress(10.0, 15.0), true);
		foodora.setMarkupPercentage(0.05);
		foodora.setDeliveryCost(5);
		foodora.setServiceFee(10);
		int theDay = new java.util.Date().getDay();
		int theMonth = new java.util.Date().getMonth() + 1 ;
		int theYear = new java.util.Date().getYear() + 1900;
		Others.Date today = new Others.Date(theYear, theMonth, theDay);
		int lastMonth = 0;
		int yearOfLastMonth = 0;
		if(theMonth == 1){
			yearOfLastMonth=today.getIntYear()-1;
			lastMonth = 12;
		}
		else{
			yearOfLastMonth=today.getIntYear();
			lastMonth=today.getIntMonth()-1;
		}
		Order order1 = new Order();
		Order order2 = new Order();
		order1.setRestaurantAttached(resto);
		order2.setRestaurantAttached(resto);
		order1.setDate(new Date(yearOfLastMonth, lastMonth, 12));
		order2.setDate(new Date(yearOfLastMonth, lastMonth, 5));
		Dish dish1 = new Dish("pat", "regular", 56);
		order1.add(dish1);
		Dish dish2 = new Dish("boulgour","glutenfree",44);
		order2.add(dish2);
		foodora.getGlobalHistoric().getListOrder().add(order2);
		foodora.getGlobalHistoric().getListOrder().add(order1);
		assertTrue(foodora.getLastMonthIncome()==100);
	}
	
	@Test
	public void testGetLastMonthNumberOfCommands() throws ExceptionUnknownStartegyType{
		MyFoodora foodora = new MyFoodora();
		Restaurant resto = new Restaurant(0245, "La tanire", "Lataniere", "contact@lataniere.fr", "1234", new Adress(10.0, 15.0), true);
		foodora.setMarkupPercentage(0.05);
		foodora.setDeliveryCost(5);
		foodora.setServiceFee(10);
		int theDay = new java.util.Date().getDay();
		int theMonth = new java.util.Date().getMonth() + 1 ;
		int theYear = new java.util.Date().getYear() + 1900;
		Others.Date today = new Others.Date(theYear, theMonth, theDay);
		int lastMonth = 0;
		int yearOfLastMonth = 0;
		if(theMonth == 1){
			yearOfLastMonth=today.getIntYear()-1;
			lastMonth = 12;
		}
		else{
			yearOfLastMonth=today.getIntYear();
			lastMonth=today.getIntMonth()-1;
		}
		Order order1 = new Order();
		Order order2 = new Order();
		order1.setRestaurantAttached(resto);
		order2.setRestaurantAttached(resto);
		order1.setDate(new Date(yearOfLastMonth, lastMonth, 12));
		order2.setDate(new Date(yearOfLastMonth, lastMonth, 5));
		Dish dish1 = new Dish("pat", "regular", 56);
		order1.add(dish1);
		Dish dish2 = new Dish("boulgour","glutenfree",44);
		order2.add(dish2);
		foodora.getGlobalHistoric().getListOrder().add(order2);
		foodora.getGlobalHistoric().getListOrder().add(order1);
		assertTrue(foodora.getNumberOfOrderLastMonth()==2);
		
	}

}
