package test;

import static org.junit.Assert.*;

import java.util.ArrayList;

import org.junit.Test;

import Core.MyFoodora;
import Exception.ExceptionUnknownDishType;
import Exception.ExceptionUnknownStartegyType;
import Exception.ExceptionUnknownTypeOfUser;
import Others.Adress;
import Others.Position;
import User.Courier;
import User.Customer;
import User.Manager;
import User.Restaurant;

public class TestCourier {
	
	public static final int UNKNOWN = 0;
	public static final int COURIER = 1;
	public static final int RESTAURANT = 2;
	public static final int CUSTOMER = 3;
	public static final int MANAGER = 4;

	@Test
	public void testSetPosition() {
		Courier courier = new Courier(0245, "martin", "martin", "martin.zebre@hotmail.fr", "1234", new Adress(10.0, 18.0), true,"25/12/1945", "martinou");
		courier.setPosition(new Position(10.0, 15.0));
		assertTrue(courier.getPosition().getX() == 10.0);
	}

	@Test
	public void testSetAvailable() {
		Courier courier = new Courier(0245, "martin", "martin", "martin.zebre@hotmail.fr", "1234", new Adress(10.0, 18.0), true,"25/12/1945", "martinou");
		courier.setAvailable();
		assertTrue(courier.isAvailable());
	}

	@Test
	public void testSetNotAvailable() {
		Courier courier = new Courier(0245, "martin", "martin", "martin.zebre@hotmail.fr", "1234", new Adress(10.0, 18.0), true,"25/12/1945", "martinou");
		courier.setNotAvailable();
		assertTrue(courier.isAvailable() == false);
	}

	@Test
	public void testRegister() throws ExceptionUnknownStartegyType {
		Courier martin = new Courier(0245, "martin", "martin", "martin.zebre@hotmail.fr", "1234", new Adress(10.0, 18.0), true,"25/12/1945", "martinou");
		MyFoodora foodora = new MyFoodora();
		martin.register(foodora);
		boolean isIn = false;
		for(Courier courier : foodora.getListCourier().getList()){
			if(courier.getName() == "martin"){
				isIn = true;
			}
		}
		
		assertTrue(isIn);
	}

	@Test
	public void testUnregister() throws ExceptionUnknownStartegyType {
		Courier martin = new Courier(0245, "martin", "martin", "martin.zebre@hotmail.fr", "1234", new Adress(10.0, 18.0), true,"25/12/1945", "martinou");
		MyFoodora foodora = new MyFoodora();
		martin.register(foodora);
		martin.unregister(foodora);
		boolean isIn = false;
		for(Courier courier : foodora.getListCourier().getList()){
			if(courier.getName() == "martin"){
				isIn = true;
			}
		}
		
		assertTrue(isIn == false);
	}

	@Test
	public void testAcceptDelivery() throws ExceptionUnknownDishType, ExceptionUnknownStartegyType, ExceptionUnknownTypeOfUser {
		MyFoodora foodora = new MyFoodora();
		Manager manager = new Manager(0674, "jacques", 	"jacquo", "jacques.martin@hotmail.fr", "123456789", new Adress(15.0, 10.0), true, "25/15/1955", "jacquo");
		manager.addUser(0145, "henry", "henry", "henry.dupont@hotmail.fr", "124578", new Adress(11.0, 15.2), true, "25/12/1994", "henrynou", foodora, CUSTOMER, "Human");
		manager.addUser(0245, "La tanire", "Lataniere", "contact@lataniere.fr", "1234", new Adress(10.0, 15.0), true,"1", "1", foodora,RESTAURANT,"Moral");
		manager.addUser(0245, "martin", "martin", "martin.zebre@hotmail.fr", "1234", new Adress(10.0, 18.0), true,"25/12/1945", "martinou", foodora,COURIER,"Human");
		Customer henry = (Customer) foodora.getUserByName("henry", "henry");
		Restaurant resto1 = (Restaurant) foodora.getUserByName("Lataniere", "La tanire");
		Courier martin = (Courier) foodora.getUserByName("martin", "martin");
		resto1.addDish("MainDish", "pat", "glutenfree", 4.5);
		ArrayList<String> command = new ArrayList<String>();
		command.add("pat");
		henry.placeOrder(resto1, "Dish", command, foodora);
		assertTrue(martin.getHistoricOrder().getListOrder().size() !=0);
	}

	@Test
	public void testGetNbOfOrderOfTheDay() throws ExceptionUnknownDishType, ExceptionUnknownStartegyType, ExceptionUnknownTypeOfUser {
		MyFoodora foodora = new MyFoodora();
		Manager manager = new Manager(0674, "jacques", 	"jacquo", "jacques.martin@hotmail.fr", "123456789", new Adress(15.0, 10.0), true, "25/15/1955", "jacquo");
		manager.addUser(0145, "henry", "henry", "henry.dupont@hotmail.fr", "124578", new Adress(11.0, 15.2), true, "25/12/1994", "henrynou", foodora, CUSTOMER, "Human");
		manager.addUser(0245, "La tanire", "Lataniere", "contact@lataniere.fr", "1234", new Adress(10.0, 15.0), true,"1", "1", foodora,RESTAURANT,"Moral");
		manager.addUser(0245, "martin", "martin", "martin.zebre@hotmail.fr", "1234", new Adress(10.0, 18.0), true,"25/12/1945", "martinou", foodora,COURIER,"Human");
		Customer henry = (Customer) foodora.getUserByName("henry", "henry");
		Restaurant resto1 = (Restaurant) foodora.getUserByName("Lataniere", "La tanire");
		Courier martin = (Courier) foodora.getUserByName("martin", "martin");
		resto1.addDish("MainDish", "pat", "glutenfree", 4.5);
		ArrayList<String> command = new ArrayList<String>();
		command.add("pat");
		henry.placeOrder(resto1, "Dish", command, foodora);
		assertTrue(martin.getNbOfOrderOfTheDay() == 1);
	}
}
