package test;

import static org.junit.Assert.*;

import java.util.ArrayList;

import org.junit.Test;

import Cards.FidelityCard;
import Core.MyFoodora;
import Exception.ExceptionUnknownDishType;
import Exception.ExceptionUnknownStartegyType;
import Exception.ExceptionUnknownTypeOfUser;
import Others.Adress;
import User.Customer;
import User.Manager;
import User.Restaurant;

public class TestCustomer {
	
	public static final int UNKNOWN = 0;
	public static final int COURIER = 1;
	public static final int RESTAURANT = 2;
	public static final int CUSTOMER = 3;
	public static final int MANAGER = 4;

	@Test
	public void testGetCard() throws ExceptionUnknownStartegyType{
		Customer henry = new Customer(0245, "henry", "henry", "henry.dupont@homtail.fr", "123456789", new Adress(10.2, 12.0), true, "25/02/1578", "henrynou");
		FidelityCard card = henry.getCard();
		assertTrue(card.returnType() == "Basic");
		
	}

	@Test
	public void testPlaceOrder() throws ExceptionUnknownStartegyType, ExceptionUnknownDishType,  ExceptionUnknownTypeOfUser {
		MyFoodora foodora = new MyFoodora();
		Manager manager = new Manager(0674, "jacques", 	"jacquo", "jacques.martin@hotmail.fr", "123456789", new Adress(15.0, 10.0), true, "25/15/1955", "jacquo");
		manager.addUser(0245, "La tanire", "Lataniere", "contact@lataniere.fr", "1234", new Adress(10.0, 15.0), true,"1", "1", foodora,RESTAURANT,"Moral");
		manager.addUser(0145, "henry", "henry", "henry.dupont@hotmail.fr", "124578", new Adress(11.0, 15.2), true, "25/12/1994", "henrynou", foodora, CUSTOMER, "Human");
		manager.addUser(0245, "martin", "martin", "martin.zebre@hotmail.fr", "1234", new Adress(10.0, 18.0), true,"25/12/1945", "martinou", foodora,COURIER,"Human");
		Customer henry = (Customer) foodora.getUserByName("henry", "henry");
		Restaurant resto1 = (Restaurant) foodora.getUserByName("Lataniere", "La tanire");
		resto1.addDish("MainDish", "pat", "glutenfree", 4.5);
		ArrayList<String> command = new ArrayList<String>();
		command.add("pat");
		henry.placeOrder(resto1, "Dish", command, foodora);
		assertTrue(henry.getHistoricOrder().getListOrder().size() == 1);
	}

	@Test
	public void testRegisterFidelityPlan() {
		Customer henry = new Customer(0245, "henry", "henry", "henry.dupont@homtail.fr", "123456789", new Adress(10.2, 12.0), true, "25/02/1578", "henrynou");
		henry.registerFidelityPlan("Point");
		FidelityCard card = henry.getCard();
		assertTrue(card.returnType() == "Point");

	}

	@Test
	public void testUnregisterFidelityPlan() {
		Customer henry = new Customer(0245, "henry", "henry", "henry.dupont@homtail.fr", "123456789", new Adress(10.2, 12.0), true, "25/02/1578", "henrynou");
		henry.registerFidelityPlan("Point");
		henry.unregisterFidelityPlan();
		FidelityCard card = henry.getCard();
		assertTrue(card.returnType() == "Basic");

	}

	@Test
	public void testGetPointInTheCard() throws ExceptionUnknownStartegyType, ExceptionUnknownDishType, ExceptionUnknownTypeOfUser {
		MyFoodora foodora = new MyFoodora();
		Manager manager = new Manager(0674, "jacques", 	"jacquo", "jacques.martin@hotmail.fr", "123456789", new Adress(15.0, 10.0), true, "25/15/1955", "jacquo");
		manager.addUser(0245, "La tanire", "Lataniere", "contact@lataniere.fr", "1234", new Adress(10.0, 15.0), true,"1", "1", foodora,RESTAURANT,"Moral");
		manager.addUser(0145, "henry", "henry", "henry.dupont@hotmail.fr", "124578", new Adress(11.0, 15.2), true, "25/12/1994", "henrynou", foodora, CUSTOMER, "Human");
		manager.addUser(0245, "martin", "martin", "martin.zebre@hotmail.fr", "1234", new Adress(10.0, 18.0), true,"25/12/1945", "martinou", foodora,COURIER,"Human");
		Customer henry = (Customer) foodora.getUserByName("henry", "henry");
		henry.registerFidelityPlan("Point");
		Restaurant resto1 = (Restaurant) foodora.getUserByName("Lataniere", "La tanire");
		resto1.addDish("MainDish", "pat", "glutenfree", 4.5);
		ArrayList<String> command = new ArrayList<String>();
		command.add("pat");
		henry.placeOrder(resto1, "Dish", command, foodora);
		assertTrue(henry.getPointInTheCard() == 1);
	}


	@Test
	public void testAcceptSpam() {
		Customer henry = new Customer(0245, "henry", "henry", "henry.dupont@homtail.fr", "123456789", new Adress(10.2, 12.0), true, "25/02/1578", "henrynou");
		henry.setSpamAgree(false);
		henry.acceptSpam();
		assertTrue(henry.isSpamAgree());
	}

	@Test
	public void testRefuseSpam() {
		Customer henry = new Customer(0245, "henry", "henry", "henry.dupont@homtail.fr", "123456789", new Adress(10.2, 12.0), true, "25/02/1578", "henrynou");
		henry.refuseSpam();
		assertTrue(henry.isSpamAgree() == false);
	}


}
