package test;

import static org.junit.Assert.*;

import org.junit.Test;

import Core.MyFoodora;

import DeliveryStrategy.ContextDeliveryStrategy;
import Exception.ExceptionUnknownStartegyType;
import Exception.ExceptionUnknownTypeOfUser;
import Others.Adress;
import Others.Position;
import User.Courier;
import User.Customer;
import User.Manager;
import User.Restaurant;

public class TestDeliveryStrategy {

	
	public static final int UNKNOWN = 0;
	public static final int COURIER = 1;
	public static final int RESTAURANT = 2;
	public static final int CUSTOMER = 3;
	public static final int MANAGER = 4;
	
	@Test
	public void testFastestDelivery() throws ExceptionUnknownStartegyType, ExceptionUnknownTypeOfUser {
		MyFoodora foodora = new MyFoodora();
		Manager manager = new Manager(0674, "jacques", 	"jacquo", "jacques.martin@hotmail.fr", "123456789", new Adress(15.0, 10.0), true, "25/15/1955", "jacquo");
		manager.addUser(0245, "martin", "martin", "martin.zebre@hotmail.fr", "1234", new Adress(1.0	, 2.0), true,"25/12/1945", "martinou", foodora,COURIER,"Human");
		manager.addUser(0246, "martin2", "martin2", "martin2.zebre@hotmail.fr", "12345", new Adress(10.0, 18.0), true,"25/12/1945", "martinou2", foodora,COURIER,"Human");
		manager.addUser(0245, "La tanire", "Lataniere", "contact@lataniere.fr", "1234", new Adress(3.0, -2.0), true,"1", "1", foodora,RESTAURANT,"Moral");
		manager.addUser(0145, "henry", "henry", "henry.dupont@hotmail.fr", "124578", new Adress(1.0, 1.0), true, "25/12/1994", "henrynou", foodora, CUSTOMER, "Human");
		Customer henry = (Customer) foodora.getUserByName("henry", "henry");
		Restaurant resto1 = (Restaurant) foodora.getUserByName("Lataniere", "La tanire");
		ContextDeliveryStrategy newStrategy = new ContextDeliveryStrategy("FastestDelivery");
		manager.setDeliveryPolicy(foodora, newStrategy);
		Courier martin = (Courier) foodora.getUserByName("martin", "martin");
		Courier martin2 = (Courier) foodora.getUserByName("martin2", "martin2");
		martin.setPosition(new Position(1.0, 2.0));
		martin2.setPosition(new Position(18.0, 20.0));
		Courier courierToChoose = foodora.getContextDeliveryPolicy().getStrategy().chooseCourier(resto1, henry, foodora.getListCourier().getList());
		assertTrue(courierToChoose.getName() == "martin");
		
		
	}
	
	@Test
	public void testFairOccupation() throws ExceptionUnknownStartegyType, ExceptionUnknownTypeOfUser{
		MyFoodora foodora = new MyFoodora();
		Manager manager = new Manager(0674, "jacques", 	"jacquo", "jacques.martin@hotmail.fr", "123456789", new Adress(15.0, 10.0), true, "25/15/1955", "jacquo");
		manager.addUser(0245, "martin", "martin", "martin.zebre@hotmail.fr", "1234", new Adress(1.0	, 2.0), true,"25/12/1945", "martinou", foodora,COURIER,"Human");
		manager.addUser(0246, "martin2", "martin2", "martin2.zebre@hotmail.fr", "12345", new Adress(10.0, 18.0), true,"25/12/1945", "martinou2", foodora,COURIER,"Human");
		manager.addUser(0245, "La tanire", "Lataniere", "contact@lataniere.fr", "1234", new Adress(3.0, -2.0), true,"1", "1", foodora,RESTAURANT,"Moral");
		manager.addUser(0145, "henry", "henry", "henry.dupont@hotmail.fr", "124578", new Adress(1.0, 1.0), true, "25/12/1994", "henrynou", foodora, CUSTOMER, "Human");
		Customer henry = (Customer) foodora.getUserByName("henry", "henry");
		Restaurant resto1 = (Restaurant) foodora.getUserByName("Lataniere", "La tanire");
		ContextDeliveryStrategy newStrategy = new ContextDeliveryStrategy("FairOccupation");
		Courier martin = (Courier) foodora.getUserByName("martin", "martin");
		martin.setNbOfLivraison(20);
		Courier courierToChoose = foodora.getContextDeliveryPolicy().getStrategy().chooseCourier(resto1, henry, foodora.getListCourier().getList());
		assertTrue(courierToChoose.getName() == "martin2");
	}

}
