package test;

import static org.junit.Assert.*;

import java.io.IOException;
import org.junit.Test;

import Exception.ExceptionUnknownDishType;
import Exception.ExceptionUnknownMealType;
import Item.Dessert;
import Item.Dish;
import Item.FactoryDish;
import Item.FactoryMeal;
import Item.Meal;

public class TestItem {

	@Test
	public void testCreateDish() throws ExceptionUnknownDishType {
		FactoryDish factory = new FactoryDish();
		Dish dish = factory.createDish("Dessert", "pat", "glutenfree", 4.5);
		assertTrue(dish instanceof Dessert);
		assertTrue(dish.getPrice() == 4.5);
	}
	
	@Test
	public void testCreateHalfMeal() throws ExceptionUnknownMealType, ExceptionUnknownDishType{
		FactoryMeal factory = new FactoryMeal();
		Meal meal = factory.createMeal("meal1", "HalfMeal");
		FactoryDish factory1 = new FactoryDish();
		Dish dish = factory1.createDish("Dessert", "pat", "glutenfree", 4.5);
		Dish dish2 = factory1.createDish("MainDish", "boulgour", "glutenfree", 5.5);
		meal.addDish(dish2);
		meal.addDish(dish);
		assertTrue(meal.getPrice() == 10.0);
	}

	@Test
	public void testCreateFullMeal() throws ExceptionUnknownDishType, ExceptionUnknownMealType{
		FactoryMeal factory = new FactoryMeal();
		Meal meal = factory.createMeal("meal1", "FullMeal");
		FactoryDish factory1 = new FactoryDish();
		Dish dish = factory1.createDish("Dessert", "pat", "glutenfree", 4.5);
		Dish dish2 = factory1.createDish("MainDish", "boulgour", "glutenfree", 5.5);
		Dish dish3 = factory1.createDish("Starter", "soupe", "glutenfree", 5);
		meal.addDish(dish3);
		meal.addDish(dish2);
		meal.addDish(dish);
		assertTrue(meal.getPrice() == 15.0);
	}
	
	
	@Test
	public void testAddDish() throws ExceptionUnknownDishType, ExceptionUnknownMealType, IOException{
		FactoryMeal factory = new FactoryMeal();
		Meal meal = factory.createMeal("meal1", "FullMeal");
		FactoryDish factory1 = new FactoryDish();
		Dish dish = factory1.createDish("Dessert", "pat", "glutenfree", 4.5);
		Dish dish2 = factory1.createDish("MainDish", "boulgour", "glutenfree", 5.5);
		Dish dish3 = factory1.createDish("Dessert", "soupe", "glutenfree", 5);
		meal.addDish(dish);
	    meal.addDish(dish2);
		meal.addDish(dish3);
		assertTrue(meal.getPrice() == 10.5);
	}
	
	
	@Test
	public void testRemoveDish() throws ExceptionUnknownMealType, ExceptionUnknownDishType{
		FactoryMeal factory = new FactoryMeal();
		Meal meal = factory.createMeal("meal1", "FullMeal");
		FactoryDish factory1 = new FactoryDish();
		Dish dish = factory1.createDish("Dessert", "pat", "glutenfree", 4.5);
		Dish dish2 = factory1.createDish("MainDish", "boulgour", "glutenfree", 5.5);
		Dish dish3 = factory1.createDish("Starter", "soupe", "glutenfree", 5);
		meal.addDish(dish);
	    meal.addDish(dish2);
		meal.addDish(dish3);
		meal.removeDish("soupe");;
		assertTrue(meal.getPrice() == 10.0);
	}
	
	
}
