package test;

import static org.junit.Assert.*;

import java.util.ArrayList;

import org.junit.Test;

import Core.MyFoodora;
import DeliveryStrategy.ContextDeliveryStrategy;
import Exception.ExceptionUnknownDishType;
import Exception.ExceptionUnknownStartegyType;
import Exception.ExceptionUnknownTypeOfUser;
import Others.Adress;
import Others.Period;
import StrategyProfit.ContextTargetProfitStrategy;
import User.Customer;
import User.Manager;
import User.Restaurant;

public class TestManager {
	
	public static final int UNKNOWN = 0;
	public static final int COURIER = 1;
	public static final int RESTAURANT = 2;
	public static final int CUSTOMER = 3;
	public static final int MANAGER = 4;

	@Test
	public void testAddUser() throws ExceptionUnknownStartegyType, ExceptionUnknownTypeOfUser {
		MyFoodora foodora = new MyFoodora();
		Manager manager = new Manager(0674, "jacques", 	"jacquo", "jacques.martin@hotmail.fr", "123456789", new Adress(15.0, 10.0), true, "25/15/1955", "jacquo");
		manager.addUser(0145, "henry", "henry", "henry.dupont@hotmail.fr", "124578", new Adress(11.0, 15.2), true, "25/12/1994", "henrynou", foodora, CUSTOMER, "Human");
		boolean isIn = false;
		for(Customer customer : foodora.getListCustomer().getList()){
			if(customer.getName() == "henry"){
				isIn = true;
			}
		}
		assertTrue(isIn);
		
		manager.addUser(0245, "La tanire", "Lataniere", "contact@lataniere.fr", "1234", new Adress(10.0, 15.0), true,null, null, foodora,RESTAURANT,"Moral");
		boolean isIn2 = false;
		for(Restaurant restaurant : foodora.getListRestaurant().getList()){
			if(restaurant.getName() == "La tanire"){
				isIn2 = true;
			}
		}
		
		assertTrue(isIn2);
		
	}

	@Test
	public void testRemoveUser() throws ExceptionUnknownStartegyType, ExceptionUnknownTypeOfUser {
		MyFoodora foodora = new MyFoodora();
		Manager manager = new Manager(0674, "jacques", 	"jacquo", "jacques.martin@hotmail.fr", "123456789", new Adress(15.0, 10.0), true, "25/15/1955", "jacquo");
		manager.addUser(0145, "henry", "henry", "henry.dupont@hotmail.fr", "124578", new Adress(11.0, 15.2), true, "25/12/1994", "henrynou", foodora, CUSTOMER, "Human");
		Customer henry = null;
		for(Customer customer : foodora.getListCustomer().getList()){
			if(customer.getName() == "henry"){
				henry = customer;
			}
		}
		manager.removeUser(henry, foodora);
		boolean isNotIn = true;
		for(Customer customer : foodora.getListCustomer().getList()){
			if(customer.getName() == "henry"){
				isNotIn = false;
			}
		}
		assertTrue(isNotIn);
	}

	@Test
	public void testChangeServiceFee() throws ExceptionUnknownStartegyType {
		MyFoodora foodora = new MyFoodora();
		Manager manager = new Manager(0674, "jacques", 	"jacquo", "jacques.martin@hotmail.fr", "123456789", new Adress(15.0, 10.0), true, "25/15/1955", "jacquo");
		manager.changeServiceFee(4.5, foodora);
		assertTrue(foodora.getServiceFee() == 4.5);
		}

	@Test
	public void testGetTotalIncome() throws ExceptionUnknownStartegyType, ExceptionUnknownDishType, ExceptionUnknownTypeOfUser {
		MyFoodora foodora = new MyFoodora();
		Manager manager = new Manager(0674, "jacques", 	"jacquo", "jacques.martin@hotmail.fr", "123456789", new Adress(15.0, 10.0), true, "25/15/1955", "jacquo");
		Others.Date begin = new Others.Date(2017, 04, 30);
		Others.Date end = new Others.Date(2018, 12, 14);
		Period period = new Period(begin, end);
		manager.addUser(0145, "henry", "henry", "henry.dupont@hotmail.fr", "124578", new Adress(11.0, 15.2), true, "25/12/1994", "henrynou", foodora, CUSTOMER, "Human");
		manager.addUser(0245, "La tanire", "Lataniere", "contact@lataniere.fr", "1234", new Adress(10.0, 15.0), true,"1", "1", foodora,RESTAURANT,"Moral");
		manager.addUser(0245, "martin", "martin", "martin.zebre@hotmail.fr", "1234", new Adress(10.0, 18.0), true,"25/12/1945", "martinou", foodora,COURIER,"Human");
		Customer henry = (Customer) foodora.getUserByName("henry", "henry");
		Restaurant resto1 = (Restaurant) foodora.getUserByName("Lataniere", "La tanire");
		resto1.addDish("MainDish", "pat", "glutenfree", 4.5);
		ArrayList<String> command = new ArrayList<String>();
		command.add("pat");
		henry.placeOrder(resto1, "Dish", command, foodora);
		double totalIncome = manager.getTotalIncome(period, foodora);
		assertTrue(totalIncome == 4.5);
	}

	@Test
	public void testGetTotalProfit() throws ExceptionUnknownStartegyType, ExceptionUnknownDishType,ExceptionUnknownTypeOfUser {
		MyFoodora foodora = new MyFoodora();
		foodora.setMarkupPercentage(0.05);
		foodora.setDeliveryCost(10);
		foodora.setServiceFee(10);
		Manager manager = new Manager(0674, "jacques", 	"jacquo", "jacques.martin@hotmail.fr", "123456789", new Adress(15.0, 10.0), true, "25/15/1955", "jacquo");
		Others.Date begin = new Others.Date(2017, 04, 30);
		Others.Date end = new Others.Date(2018, 12, 14);
		Period period = new Period(begin, end);
		manager.addUser(0145, "henry", "henry", "henry.dupont@hotmail.fr", "124578", new Adress(11.0, 15.2), true, "25/12/1994", "henrynou", foodora, CUSTOMER, "Human");
		manager.addUser(0245, "La tanire", "Lataniere", "contact@lataniere.fr", "1234", new Adress(10.0, 15.0), true,"1", "1", foodora,RESTAURANT,"Moral");
		manager.addUser(0245, "martin", "martin", "martin.zebre@hotmail.fr", "1234", new Adress(10.0, 18.0), true,"25/12/1945", "martinou", foodora,COURIER,"Human");
		Customer henry = (Customer) foodora.getUserByName("henry", "henry");
		Restaurant resto1 = (Restaurant) foodora.getUserByName("Lataniere", "La tanire");
		resto1.addDish("MainDish", "pat", "glutenfree", 100);
		ArrayList<String> command = new ArrayList<String>();
		command.add("pat");
		henry.placeOrder(resto1, "Dish", command, foodora);
		double totalProfit = manager.getTotalProfit(period, foodora);
		assertTrue(totalProfit == 5.);
	}

	@Test
	public void testGetAverageIncomePerCustomer() throws ExceptionUnknownStartegyType, ExceptionUnknownDishType, ExceptionUnknownTypeOfUser {
		MyFoodora foodora = new MyFoodora();
		Manager manager = new Manager(0674, "jacques", 	"jacquo", "jacques.martin@hotmail.fr", "123456789", new Adress(15.0, 10.0), true, "25/15/1955", "jacquo");
		Others.Date begin = new Others.Date(2017, 04, 30);
		Others.Date end = new Others.Date(2018, 12, 14);
		Period period = new Period(begin, end);
		manager.addUser(0145, "henry", "henry", "henry.dupont@hotmail.fr", "124578", new Adress(11.0, 15.2), true, "25/12/1994", "henrynou", foodora, CUSTOMER, "Human");
		manager.addUser(0146, "henry2", "henry2", "henry2.dupont@hotmail.fr", "1245789", new Adress(11.0, 15.3), true, "25/12/1995", "henrynou2", foodora, CUSTOMER, "Human");
		manager.addUser(0245, "La tanire", "Lataniere", "contact@lataniere.fr", "1234", new Adress(10.0, 15.0), true,"1", "1", foodora,RESTAURANT,"Moral");
		manager.addUser(0245, "martin", "martin", "martin.zebre@hotmail.fr", "1234", new Adress(10.0, 18.0), true,"25/12/1945", "martinou", foodora,COURIER,"Human");
		Customer henry = (Customer) foodora.getUserByName("henry", "henry");
		Restaurant resto1 = (Restaurant) foodora.getUserByName("Lataniere", "La tanire");
		resto1.addDish("MainDish", "pat", "glutenfree", 5.0);
		ArrayList<String> command = new ArrayList<String>();
		command.add("pat");
		henry.placeOrder(resto1, "Dish", command, foodora);
		double averageIncomePerCustomer = manager.getAverageIncomePerCustomer(foodora, period);
		assertTrue(averageIncomePerCustomer == 5.0);
	}

	@Test
	public void testSetTargetProfitPolicy() throws ExceptionUnknownStartegyType {
		MyFoodora foodora = new MyFoodora();
		Manager manager = new Manager(0674, "jacques", 	"jacquo", "jacques.martin@hotmail.fr", "123456789", new Adress(15.0, 10.0), true, "25/15/1955", "jacquo");
		ContextTargetProfitStrategy newStrategy = new ContextTargetProfitStrategy("ServiceFee");
		manager.setTargetProfitPolicy(foodora, newStrategy);
		assertTrue(foodora.getContextTargetProfitStrategy().toString() == "ServiceFee");
	}

	@Test
	public void testSetDeliveryPolicy() throws ExceptionUnknownStartegyType {
		MyFoodora foodora = new MyFoodora();
		Manager manager = new Manager(0674, "jacques", 	"jacquo", "jacques.martin@hotmail.fr", "123456789", new Adress(15.0, 10.0), true, "25/15/1955", "jacquo");
		ContextDeliveryStrategy newStrategy = new ContextDeliveryStrategy("FastestDelivery");
		manager.setDeliveryPolicy(foodora, newStrategy);
		assertTrue(foodora.getContextDeliveryPolicy().toString() == "FastestDelivery");	
	}

	@Test
	public void testGetMostSellingRestaurant() throws ExceptionUnknownStartegyType, ExceptionUnknownDishType, ExceptionUnknownTypeOfUser {
		MyFoodora foodora = new MyFoodora();
		Manager manager = new Manager(0674, "jacques", 	"jacquo", "jacques.martin@hotmail.fr", "123456789", new Adress(15.0, 10.0), true, "25/15/1955", "jacquo");
		manager.addUser(0245, "La tanire", "Lataniere", "contact@lataniere.fr", "1234", new Adress(10.0, 15.0), true,"1", "1", foodora,RESTAURANT,"Moral");
		manager.addUser(0245, "La tanire2", "Lataniere2", "contact@lataniere2.fr", "12345", new Adress(10.0, 18.0), true,null, null, foodora,RESTAURANT,"Moral");
		manager.addUser(0145, "henry", "henry", "henry.dupont@hotmail.fr", "124578", new Adress(11.0, 15.2), true, "25/12/1994", "henrynou", foodora, CUSTOMER, "Human");
		manager.addUser(0245, "martin", "martin", "martin.zebre@hotmail.fr", "1234", new Adress(10.0, 18.0), true,"25/12/1945", "martinou", foodora,COURIER,"Human");
		Customer henry = (Customer) foodora.getUserByName("henry", "henry");
		Restaurant resto1 = (Restaurant) foodora.getUserByName("Lataniere", "La tanire");
		resto1.addDish("MainDish", "pat", "glutenfree", 4.5);
		ArrayList<String> command = new ArrayList<String>();
		command.add("pat");
		henry.placeOrder(resto1, "Dish", command, foodora);
		String mostSellingOne = manager.getMostSellingRestaurant(foodora);
		assertTrue(mostSellingOne == "La tanire");
	}

	@Test
	public void testGetLeastSellingRestaurant() throws ExceptionUnknownStartegyType, ExceptionUnknownDishType, ExceptionUnknownTypeOfUser {
		MyFoodora foodora = new MyFoodora();
		Manager manager = new Manager(0674, "jacques", 	"jacquo", "jacques.martin@hotmail.fr", "123456789", new Adress(15.0, 10.0), true, "25/15/1955", "jacquo");
		manager.addUser(0245, "La tanire", "Lataniere", "contact@lataniere.fr", "1234", new Adress(10.0, 15.0), true,"1", "1", foodora,RESTAURANT,"Moral");
		manager.addUser(0245, "La tanire2", "Lataniere2", "contact@lataniere2.fr", "12345", new Adress(10.0, 18.0), true,null, null, foodora,RESTAURANT,"Moral");
		manager.addUser(0145, "henry", "henry", "henry.dupont@hotmail.fr", "124578", new Adress(11.0, 15.2), true, "25/12/1994", "henrynou", foodora, CUSTOMER, "Human");
		manager.addUser(0245, "martin", "martin", "martin.zebre@hotmail.fr", "1234", new Adress(10.0, 18.0), true,"25/12/1945", "martinou", foodora,COURIER,"Human");
		Customer henry = (Customer) foodora.getUserByName("henry", "henry");
		Restaurant resto1 = (Restaurant) foodora.getUserByName("Lataniere", "La tanire");
		resto1.addDish("MainDish", "pat", "glutenfree", 4.5);
		ArrayList<String> command = new ArrayList<String>();
		command.add("pat");
		henry.placeOrder(resto1, "Dish", command, foodora);
		String mostSellingOne = manager.getLeastSellingRestaurant(foodora);
		assertTrue(mostSellingOne == "La tanire2");
	}

	@Test
	public void testGetLeastActiveCourier() throws ExceptionUnknownStartegyType, ExceptionUnknownDishType, ExceptionUnknownTypeOfUser {
		MyFoodora foodora = new MyFoodora();
		Manager manager = new Manager(0674, "jacques", 	"jacquo", "jacques.martin@hotmail.fr", "123456789", new Adress(15.0, 10.0), true, "25/15/1955", "jacquo");
		manager.addUser(0245, "martin", "martin", "martin.zebre@hotmail.fr", "1234", new Adress(10.0, 18.0), true,"25/12/1945", "martinou", foodora,COURIER,"Human");
		manager.addUser(0246, "martin2", "martin2", "martin2.zebre@hotmail.fr", "12345", new Adress(10.0, 18.0), true,"25/12/1945", "martinou2", foodora,COURIER,"Human");
		manager.addUser(0145, "henry", "henry", "henry.dupont@hotmail.fr", "124578", new Adress(11.0, 15.2), true, "25/12/1994", "henrynou", foodora, CUSTOMER, "Human");
		manager.addUser(0245, "La tanire", "Lataniere", "contact@lataniere.fr", "1234", new Adress(10.0, 15.0), true,"1", "1", foodora,RESTAURANT,"Moral");
		Customer henry = (Customer) foodora.getUserByName("henry", "henry");
		Restaurant resto1 = (Restaurant) foodora.getUserByName("Lataniere", "La tanire");
		resto1.addDish("MainDish", "pat", "glutenfree", 4.5);
		ArrayList<String> command = new ArrayList<String>();
		command.add("pat");
		henry.placeOrder(resto1, "Dish", command, foodora);
		String leastActiveCourier = manager.getLeastActiveCourier(foodora);
		assertTrue(leastActiveCourier == "martin");
	}

	@Test
	public void testGetMostActiveCourier() throws ExceptionUnknownStartegyType, ExceptionUnknownDishType, ExceptionUnknownTypeOfUser {
		MyFoodora foodora = new MyFoodora();
		Manager manager = new Manager(0674, "jacques", 	"jacquo", "jacques.martin@hotmail.fr", "123456789", new Adress(15.0, 10.0), true, "25/15/1955", "jacquo");
		manager.addUser(0245, "martin", "martin", "martin.zebre@hotmail.fr", "1234", new Adress(10.0, 18.0), true,"25/12/1945", "martinou", foodora,COURIER,"Human");
		manager.addUser(0246, "martin2", "martin2", "martin2.zebre@hotmail.fr", "12345", new Adress(10.0, 18.0), true,"25/12/1945", "martinou2", foodora,COURIER,"Human");
		manager.addUser(0145, "henry", "henry", "henry.dupont@hotmail.fr", "124578", new Adress(11.0, 15.2), true, "25/12/1994", "henrynou", foodora, CUSTOMER, "Human");
		manager.addUser(0245, "La tanire", "Lataniere", "contact@lataniere.fr", "1234", new Adress(10.0, 15.0), true,"1", "1", foodora,RESTAURANT,"Moral");
		Customer henry = (Customer) foodora.getUserByName("henry", "henry");
		Restaurant resto1 = (Restaurant) foodora.getUserByName("Lataniere", "La tanire");
		resto1.addDish("MainDish", "pat", "glutenfree", 4.5);
		ArrayList<String> command = new ArrayList<String>();
		command.add("pat");
		henry.placeOrder(resto1, "Dish", command, foodora);
		String mostActiveCourier = manager.getMostActiveCourier(foodora);
		assertTrue(mostActiveCourier == "martin2");
	}

	@Test
	public void testActivateUser() throws ExceptionUnknownStartegyType, ExceptionUnknownTypeOfUser {
		MyFoodora foodora = new MyFoodora();
		Manager manager = new Manager(0674, "jacques", 	"jacquo", "jacques.martin@hotmail.fr", "123456789", new Adress(15.0, 10.0), true, "25/15/1955", "jacquo");
		manager.addUser(0145, "henry", "henry", "henry.dupont@hotmail.fr", "124578", new Adress(11.0, 15.2), true, "25/12/1994", "henrynou", foodora, CUSTOMER, "Human");
		Customer henry = (Customer) foodora.getUserByName("henry", "henry");
		henry.setActivated(false);
		manager.activateUser(foodora, henry);
		assertTrue(henry.isActivated());
	}

	@Test
	public void testDesactivateUser() throws ExceptionUnknownStartegyType, ExceptionUnknownTypeOfUser {
		MyFoodora foodora = new MyFoodora();
		Manager manager = new Manager(0674, "jacques", 	"jacquo", "jacques.martin@hotmail.fr", "123456789", new Adress(15.0, 10.0), true, "25/15/1955", "jacquo");
		manager.addUser(0145, "henry", "henry", "henry.dupont@hotmail.fr", "124578", new Adress(11.0, 15.2), true, "25/12/1994", "henrynou", foodora, CUSTOMER, "Human");
		Customer henry = (Customer) foodora.getUserByName("henry", "henry");
		henry.setActivated(true);
		manager.desactivateUser(foodora, henry);
		assertTrue(henry.isActivated() == false);	
		
	}

}
