package test;

import static org.junit.Assert.*;

import org.junit.Test;

import Core.MyFoodora;
import Exception.ExceptionUnknownDishType;
import Exception.ExceptionUnknownMealType;
import Exception.ExceptionUnknownStartegyType;
import Exception.ExceptionUnknownTypeOfUser;
import Item.Dish;
import Item.FactoryDish;
import Item.FactoryMeal;
import Item.Meal;
import Offers.MealOfTheWeek;
import Order.Order;
import Others.Adress;
import User.Customer;
import User.Manager;
import User.Restaurant;

public class TestOffers {
	
	public static final int UNKNOWN = 0;
	public static final int COURIER = 1;
	public static final int RESTAURANT = 2;
	public static final int CUSTOMER = 3;
	public static final int MANAGER = 4;


	@Test
	public void testNotifyObservers() throws ExceptionUnknownMealType, ExceptionUnknownDishType {
		Customer customer = new Customer(0145, "henry", "henry", "henry.dupont@hotmail.fr", "124578", new Adress(11.0, 15.2), true, "25/12/1994", "henrynou");
		Restaurant r = new Restaurant(0245, "La tanire", "Lataniere", "contact@lataniere.fr", "1234", new Adress(10.0, 15.0), true);
		Order order = new Order();
		FactoryMeal factory = new FactoryMeal();
		Meal meal = factory.createMeal("meal1", "FullMeal");
		FactoryDish factory1 = new FactoryDish();
		Dish dish = factory1.createDish("Dessert", "pat", "glutenfree", 4.5);
		Dish dish2 = factory1.createDish("MainDish", "boulgour", "glutenfree", 5.5);
		Dish dish3 = factory1.createDish("Starter", "soupe", "glutenfree", 5);
		meal.addDish(dish3);
		meal.addDish(dish2);
		meal.addDish(dish);
		MealOfTheWeek mealOfTheWeek = new MealOfTheWeek(meal);
		mealOfTheWeek.attachObserver(customer);
		mealOfTheWeek.setInterests(r);
		assertTrue(customer.getNbOfOfferReceived() == 1);
	}
	
	@Test
	public void testInFoodora() throws ExceptionUnknownDishType, ExceptionUnknownStartegyType, ExceptionUnknownMealType, ExceptionUnknownTypeOfUser{
		MyFoodora foodora = new MyFoodora();
		Manager manager = new Manager(0674, "jacques", 	"jacquo", "jacques.martin@hotmail.fr", "123456789", new Adress(15.0, 10.0), true, "25/15/1955", "jacquo");
		manager.addUser(0145, "henry", "henry", "henry.dupont@hotmail.fr", "124578", new Adress(11.0, 15.2), true, "25/12/1994", "henrynou", foodora, CUSTOMER, "Human");
		manager.addUser(0146, "henry2", "henry2", "henry2.dupont@hotmail.fr", "1245789", new Adress(11.0, 15.3), true, "25/12/1995", "henrynou2", foodora, CUSTOMER, "Human");
		manager.addUser(0245, "La tanire", "Lataniere", "contact@lataniere.fr", "1234", new Adress(10.0, 15.0), true,"1", "1", foodora,RESTAURANT,"Moral");
		Customer henry = (Customer) foodora.getUserByName("henry", "henry");
		Customer henry2 = (Customer) foodora.getUserByName("henry2", "henry2");
		henry2.refuseSpam();
		Restaurant resto = (Restaurant) foodora.getUserByName("Lataniere", "La tanire");
		resto.addDish("MainDish", "pat", "glutenfree", 4.5);
		resto.addDish("Starter","boulgour","vegetarian", 5);
		resto.addDish("Dessert", "fondant", "regular", 7);
		Dish pat = resto.getDishByName("pat");
		Dish boulgour = resto.getDishByName("boulgour");
		Dish fondant = resto.getDishByName("fondant");
		resto.createMeal("meal1", "FullMeal");
		Meal meal1 = resto.getMealByName("meal1");
		resto.addDishToMeal(fondant, meal1);
		resto.addDishToMeal(boulgour, meal1);
		resto.addDishToMeal(pat, meal1);
		resto.addMealOfTheWeek(foodora, "meal1");
		assertTrue(henry.getNbOfOfferReceived() ==1);
		assertTrue(henry2.getNbOfOfferReceived() == 0);
	}

}
