package test;

import static org.junit.Assert.*;

import java.util.ArrayList;

import org.junit.Test;

import Core.MyFoodora;
import Exception.ExceptionUnknownDishType;
import Exception.ExceptionUnknownMealType;
import Exception.ExceptionUnknownStartegyType;
import Exception.ExceptionUnknownTypeOfUser;
import Item.Dish;
import Item.HalfMeal;
import Item.Meal;
import Others.Adress;
import User.Manager;
import User.Restaurant;

public class TestRestaurant {
	
	public static final int UNKNOWN = 0;
	public static final int COURIER = 1;
	public static final int RESTAURANT = 2;
	public static final int CUSTOMER = 3;
	public static final int MANAGER = 4;

	@Test
	public void testAddMeal() throws ExceptionUnknownDishType, ExceptionUnknownMealType {
		Restaurant resto = new Restaurant(0245, "La tanire", "Lataniere", "contact@lataniere.fr", "1234", new Adress(10.0, 15.0), true);
		resto.addDish("MainDish", "pat", "glutenfree", 4.5);
		resto.addDish("Starter","boulgour","vegetarian", 5);
		resto.addDish("Dessert", "fondant", "regular", 7);
		Dish pat = resto.getDishByName("pat");
		Dish boulgour = resto.getDishByName("boulgour");
		Dish fondant = resto.getDishByName("fondant");
		resto.createMeal("meal1", "FullMeal");
		Meal meal1 = resto.getMealByName("meal1");
		resto.addDishToMeal(fondant, meal1);
		resto.addDishToMeal(boulgour, meal1);
		resto.addDishToMeal(pat, meal1);
		boolean isIn = false;
		for(Meal m : resto.getListOfMeal()){
			if(m.getName() == "meal1"){
				isIn = true;
			}
		}
		assertTrue(isIn);
		
	}

	@Test
	public void testAddDish() throws ExceptionUnknownDishType {
		Restaurant resto = new Restaurant(0245, "La tanire", "Lataniere", "contact@lataniere.fr", "1234", new Adress(10.0, 15.0), true);
		resto.addDish("MainDish", "pat", "glutenfree", 4.5);
		boolean isIn = false;
		for(Dish d : resto.getListOfDish()){
			if(d.getName() == "pat"){
				isIn = true;
			}
		}
		
		assertTrue(isIn);
	}

	@Test
	public void testRemoveMeal() throws ExceptionUnknownDishType, ExceptionUnknownMealType {
		Restaurant resto = new Restaurant(0245, "La tanire", "Lataniere", "contact@lataniere.fr", "1234", new Adress(10.0, 15.0), true);
		resto.addDish("MainDish", "pat", "glutenfree", 4.5);
		resto.addDish("Starter","boulgour","vegetarian", 5);
		resto.addDish("Dessert", "fondant", "regular", 7);
		Dish pat = resto.getDishByName("pat");
		Dish boulgour = resto.getDishByName("boulgour");
		Dish fondant = resto.getDishByName("fondant");
		resto.createMeal("meal1", "FullMeal");
		Meal meal1 = resto.getMealByName("meal1");
		resto.addDishToMeal(fondant, meal1);
		resto.addDishToMeal(boulgour, meal1);
		resto.addDishToMeal(pat, meal1);
		resto.removeMeal(meal1);
		boolean isIn = false;
		for(Meal m : resto.getListOfMeal()){
			if(m.getName() == "meal1"){
				isIn = true;
			}
		}
		assert(isIn == false);
	}

	@Test
	public void testRemoveDish() throws ExceptionUnknownDishType {
		Restaurant resto = new Restaurant(0245, "La tanire", "Lataniere", "contact@lataniere.fr", "1234", new Adress(10.0, 15.0), true);
		resto.addDish("MainDish", "pat", "glutenfree", 4.5);
		Dish pat = resto.getDishByName("pat");
		resto.removeDish(pat);
		boolean isIn = false;
		for(Dish d : resto.getListOfDish()){
			if(d.getName() == "pat"){
				isIn = true;
			}
		}
		assertTrue(isIn == false);
	}

	@Test
	public void testGetSortListOfHalfMeal() throws ExceptionUnknownDishType, ExceptionUnknownMealType {
		Restaurant resto = new Restaurant(0245, "La tanire", "Lataniere", "contact@lataniere.fr", "1234", new Adress(10.0, 15.0), true);
		resto.addDish("MainDish", "pat", "glutenfree", 4.5);
		resto.addDish("Starter","boulgour","vegetarian", 5);
		resto.addDish("Dessert", "fondant", "regular", 7);
		Dish pat = resto.getDishByName("pat");
		Dish boulgour = resto.getDishByName("boulgour");
		Dish fondant = resto.getDishByName("fondant");
		resto.createMeal("meal1", "HalfMeal");
		Meal meal1 = resto.getMealByName("meal1");
		resto.addDishToMeal(fondant, meal1);
		resto.addDishToMeal(pat, meal1);
		resto.createMeal("meal2", "HalfMeal");
		Meal meal2 = resto.getMealByName("meal2");
		resto.addDishToMeal(boulgour, meal2);
		resto.addDishToMeal(pat, meal2);
		for(Meal m : resto.getListOfMeal()){
			if(m.getName() == "meal1"){
				m.setNbOfSell(20);
			}
		}
		ArrayList<HalfMeal> sortList = resto.getSortListOfHalfMeal();
		HalfMeal mostSelled = sortList.get(0);
		assertTrue(mostSelled.getName() == "meal1");
	}

	@Test
	public void testGetSortListOfDish() throws ExceptionUnknownDishType, ExceptionUnknownStartegyType, ExceptionUnknownTypeOfUser {
		MyFoodora foodora = new MyFoodora();
		Manager manager = new Manager(0674, "jacques", 	"jacquo", "jacques.martin@hotmail.fr", "123456789", new Adress(15.0, 10.0), true, "25/15/1955", "jacquo");
		manager.addUser(0245, "La tanire", "Lataniere", "contact@lataniere.fr", "1234", new Adress(10.0, 15.0), true,"1", "1", foodora,RESTAURANT,"Moral");
		Restaurant resto1 = (Restaurant) foodora.getUserByName("Lataniere", "La tanire");
		resto1.addDish("MainDish", "pat", "glutenfree", 4.5);
		resto1.addDish("Starter","boulgour","vegetarian", 5);
		ArrayList<Dish> cartOfRestaurant = resto1.getListOfDish();
		for(Dish dish : cartOfRestaurant){
			if(dish.getName() == "pat"){
				dish.setNbOfSell(20);
			}
		}
		ArrayList<Dish> sortList = resto1.getSortListOfDish();
		Dish mostSelled = sortList.get(0);
		assertTrue(mostSelled.getName() == "pat");
		
	}
	
	@Test
	public void testSetMealOfTheWeek() throws ExceptionUnknownStartegyType, ExceptionUnknownMealType, ExceptionUnknownDishType, ExceptionUnknownTypeOfUser{
		MyFoodora foodora = new MyFoodora();
		Manager manager = new Manager(0674, "jacques", 	"jacquo", "jacques.martin@hotmail.fr", "123456789", new Adress(15.0, 10.0), true, "25/15/1955", "jacquo");
		manager.addUser(0245, "La tanire", "Lataniere", "contact@lataniere.fr", "1234", new Adress(10.0, 15.0), true,"1", "1", foodora,RESTAURANT,"Moral");
		Restaurant resto1 = (Restaurant) foodora.getUserByName("Lataniere", "La tanire");
		Restaurant resto = (Restaurant) foodora.getUserByName("Lataniere", "La tanire");
		resto.addDish("MainDish", "pat", "glutenfree", 4.5);
		resto.addDish("Starter","boulgour","vegetarian", 5);
		resto.addDish("Dessert", "fondant", "regular", 7);
		Dish pat = resto.getDishByName("pat");
		Dish boulgour = resto.getDishByName("boulgour");
		Dish fondant = resto.getDishByName("fondant");
		resto.createMeal("meal1", "FullMeal");
		Meal meal1 = resto.getMealByName("meal1");
		resto.addDishToMeal(fondant, meal1);
		resto.addDishToMeal(boulgour, meal1);
		resto.addDishToMeal(pat, meal1);
		resto.addMealOfTheWeek(foodora, "meal1");
		assertTrue(resto.getMealOfTheWeek().getMeal().getName() == "meal1");
	}

}
